# -*- coding: utf-8 -*-
from __future__ import absolute_import

import logging


def create_logger(name):
    """
    """
    logger = logging.getLogger()
    handler = logging.StreamHandler()
    formatter = logging.Formatter(
        '%(levelname)s:%(funcName)s(%(lineno)s): %(message)s')
    handler.setFormatter(formatter)
    logger.addHandler(handler)
    logger.setLevel(logging.INFO)
    return logger



# -*- coding: utf-8 -*-

#  This software and supporting documentation are distributed by
#      Institut Federatif de Recherche 49
#      CEA/NeuroSpin, Batiment 145,
#      91191 Gif-sur-Yvette cedex
#      France
#
# This software is governed by the CeCILL-B license under
# French law and abiding by the rules of distribution of free software.
# You can  use, modify and/or redistribute the software under the
# terms of the CeCILL-B license as circulated by CEA, CNRS
# and INRIA at the following URL "http://www.cecill.info".
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL-B license and that you accept its terms.

'''
This module is useful whenever you need to be sure that a function (more
exactly anything that can be called by python) is executed in a particular
thread.

For exemple, if you use PyMat (which is an interface between Python and a Matlab,
see U{http://claymore.engineer.gvsu.edu/~steriana/Python} for more information)
it is necessary that all calls to pymat.eval are done by the same thread that
called pymat.open. If you want to use PyMat in a multi-threaded application,
you will have to build appropriate calling mechanism as the one proposed in
this module.

The main idea is to use a list containing functions to be called with their
parameters. A single thread is used to get entries from the list and execute
the corresponding functions. Any thread can put a call request on the list
either asynchonously (the requesting thread continues to run without waiting
for the call to be done) or synchronously (the requesting thread is stopped
until the call is done and the result available).

* author: Yann Cointepas
* organization: NeuroSpin
* license: `CeCILL B <http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html>`_
'''
from __future__ import absolute_import
__docformat__ = "restructuredtext en"

import threading
import time


class SingleThreadCalls(object):

    '''
    Allows the registration of functions that are going to be called from a
    single thread. This single thread must continuously call
    :meth:`processFunctions`.

    Registration can be blocking (see :meth:`call`) or non blocking (see
    :meth:`push`).

    Blocking registration waits for the function to be processed and returns
    its result (or raises its exception). Non blocking registration puts the
    function in the list and returns immediately, ignoring any return value or
    exception.

    Example::

        stc = SingleThreadCall()
        processingThread = threading.Thread(target=stc.processingLoop)
        stc.setProcessingThread(processingThread)
        processingThread.start()
    '''

    def __init__(self, thread=None):
        '''
        The thread passed in parameter is the processing thread of this
        SingleThreadCalls. When started (which is not been done by
        :class:`SingleThreadCall`) it must continuously call
        :meth:`processFunctions` to execute the functions that have been
        registered with :meth:`call` and :meth:`push`.

        Parameters
        ----------
        thread: :class:`threading.Thread` instance or *None*
            Processing thread. If *None*, :func:`threading.current_thread`
            is used.
        '''
        self._queue = []
        if thread is None:
            thread = threading.current_thread()
        self._thread = thread
        self._condition = threading.Condition()

    def setProcessingThread(self, thread):
        '''
        Defines the thread that processes the functions. The behaviour of
        :meth:`call` and :meth:`push` is different if called from the
        processing thread or from another one.

        Parameters
        ----------
        thread: :class:`threading.Thread`
            Processing thread
        '''
        self._condition.acquire()
        try:
            self._thread = thread
        finally:
            self._condition.release()

    def call(self, function, *args, **kwargs):
        '''
        Registers a function call and waits for the processing thread to call
        it.

        Returns the result of the function. If an exception is raised during
        the execution of the function, this exception is also raised by
        :meth:`call`.
        Therefore :meth:`call` behaves like a direct call to the function.

        It is possible to use :meth:`call` from the processing thread. In this
        case, no registration is done (the function is executed immedialey).

        Parameters
        ----------
        function: callable
            function to execute
        args:
            parameters of the function
        kwargs:
            keyword parameters of the function

        Returns
        -------
        any type
            the result of the function call
        '''
        if threading.current_thread() is self._thread:
            result = function(*args, **kwargs)
        else:
            semaphore = threading.Semaphore(0)
            semaphore._result = None
            semaphore._exception = None
            self._condition.acquire()
            try:
                self._queue.append(
                    (self._executeAndNotify, (semaphore, function, args, kwargs), {}))
                self._condition.notify()
            finally:
                self._condition.release()
            semaphore.acquire()
            if semaphore._exception is not None:
                e = semaphore._exception
                semaphore.release()
                raise e
            result = semaphore._result
        return result

    def _executeAndNotify(semaphore, function, args, kwargs):
        try:
            result = function(*args, **kwargs)
            semaphore._result = result
            semaphore._exception = None
            semaphore.release()
        except Exception as e:
            semaphore._exception = e
            semaphore.release()
    _executeAndNotify = staticmethod(_executeAndNotify)

    def push(self, function, *args, **kwargs):
        '''
        Same as the :meth:`call` method but always puts the function on the
        queue and returns immediatly. If :meth:`push` is called from the
        processing thread, the function is called immediately (*i.e.*
        synchronously).

        Parameters
        ----------
        function: callable
            function to execute
        args:
            parameters of the function
        kwargs:
            keyword parameters of the function

        Returns
        -------
        None:
            push() does not return anything.
        '''
        if threading.current_thread() is self._thread:
            function(*args, **kwargs)
        else:
            self._condition.acquire()
            try:
                self._queue.append((function, args, kwargs))
                self._condition.notify()
            finally:
                self._condition.release()

    def stop(self):
        '''
        Tells the processing thread to finish the processing of functions in the
        queue and then stop all processing. This call pushes a special value on
        the function list. All functions that are on the list before this value
        will be processed but functions registered after the special value will be
        ignored.
        '''
        self._condition.acquire()
        try:
            self._queue.append(None)
            self._condition.notify()
        finally:
            self._condition.release()

    def processFunctions(self, blocking=False):
        '''
        This method extracts all functions (along with their parameters) from
        the queue and execute them. It returns the number of function executed
        *None* if :meth:`self.stop() <stop>` has been called (in that case the
        processing loop must end).

        The processing thread must continuoulsy call this method until *None*
        is returned.

        .. seealso:: :meth:`processingLoop`

        Parameters
        ----------
        blocking: bool
            if *blocking* is *False* (the default), *processFunctions*

        Returns
        -------
        int:
            * Returns 0 immediately if it cannot access the function list (for
              example if it is locked by another thread that is registering a
              function).
            * If *blocking* is *True*, *processFunctions* waits until the
              function list is available, and returns the number of function
              called
            * If :meth:`stop` has been called, *None* is returned
        '''
        if self._condition.acquire(blocking):
            try:
                actions = self._queue
                self._queue = []
            finally:
                self._condition.release()
            result = 0
            for action in actions:
                if action is None:
                    return None
                function, args, kwargs = action
                function(*args, **kwargs)
                result += 1
            return result
        return 0

    def processingLoop(self):
        '''
        Continuously executes :meth:`processFunctions` until it returns *None*

        .. seealso:: :meth:`processFunctions`
        '''
        actionCount = 0
        self._condition.acquire()
        self.processFunctions()
        while actionCount is not None:
            self._condition.wait()
            actionCount = self.processFunctions()
        self._condition.release()



# -*- coding: utf-8 -*-
#
# SOMA - Copyright (C) CEA, 2015
# Distributed under the terms of the CeCILL-B license, as published by
# the CEA-CNRS-INRIA. Refer to the LICENSE file or to
# http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html
# for details.
#

# System import
from __future__ import absolute_import
import logging
from functools import partial
import sys
import six

# Define the logger
logger = logging.getLogger(__name__)

# Soma import
from soma.qt_gui.qt_backend import Qt
from soma.utils.functiontools import SomaPartial
from soma.utils.weak_proxy import weak_proxy
from soma.controller.trait_utils import trait_ids
import traits.api as traits
import sip


class CompoundControlWidget(object):

    """ Control to select a value from compound/either traits.
    """

    @staticmethod
    def is_valid(control_instance, *args, **kwargs):
        """ Method to check if the new control value is correct.

        Parameters
        ----------
        control_instance: QWidget (mandatory)
            the control widget we want to validate

        Returns
        -------
        out: bool
        """
        # nothing to do here since we delegate to another control widget.
        return True

    @classmethod
    def check(cls, control_instance):
        """ Check if a controller widget control is filled correctly.

        Parameters
        ----------
        cls: EnumControlWidget (mandatory)
            an EnumControlWidget control
        control_instance: QComboBox (mandatory)
            the control widget we want to validate
        """
        # Hook: function that will be called to check for typo
        # when a 'textEdited' qt signal is emited
        widget_callback = partial(cls.is_valid, weak_proxy(control_instance))

        # Execute manually the first time the control check method
        widget_callback()

    @staticmethod
    def add_callback(callback, control_instance):
        """ Method to add a callback to the control instance when a 'editTextChanged'
        signal is emited.

        Parameters
        ----------
        callback: @function (mandatory)
            the function that will be called when a 'editTextChanged' signal is
            emited.
        control_instance: QComboBox (mandatory)
            the control widget we want to validate
        """
        # nothing to do here since we delegate to another control widget.
        pass

    @staticmethod
    def create_widget(parent, control_name, control_value, trait,
                      label_class=None, user_data=None):
        """ Create the widget.

        Parameters
        ----------
        parent: QWidget (mandatory)
            the parent widget
        control_name: str (mandatory)
            the name of the control we want to create
        control_value: str (mandatory)
            the default control value, here the enum values
        trait: Tait (mandatory)
            the trait associated to the control
        label_class: Qt widget class (optional, default: None)
            the label widget will be an instance of this class. Its constructor
            will be called using 2 arguments: the label string and the parent
            widget.

        Returns
        -------
        out: 2-uplet
            a two element tuple of the form (control widget: QComboBox,
            associated label: QLabel)
        """
        # Create the widget that will be used to select a value
        widget = Qt.QWidget(parent)

        # we have a combobox for the trait type, and one of the control types
        # implementations depending on it

        layout = Qt.QVBoxLayout()
        widget.setLayout(layout)
        widget.type_combo = Qt.QComboBox()
        hlayout = Qt.QHBoxLayout()
        #layout.addLayout(hlayout)
        lwidget = Qt.QWidget()
        lwidget.setLayout(hlayout)
        hlayout.addWidget(Qt.QLabel('Compound type:'))
        hlayout.addWidget(widget.type_combo)
        widget.header_widget = lwidget
        widget.user_data = user_data

        # get compound types
        types = trait_ids(trait)
        for t in types:
            widget.type_combo.addItem(t)
        widget.type_combo.setCurrentIndex(0)

        widget.compound_widget = None
        widget.trait = trait  # we need to access it later
        widget.trait_name = control_name
        widget.compound_label = None

        type_id = CompoundControlWidget.type_id_for(trait.handler.handlers,
                                                    control_value)
        widget.current_type_id = type_id
        widget.type_combo.setCurrentIndex(type_id)

        CompoundControlWidget.create_compound_widget(widget)

        widget.type_combo.currentIndexChanged.connect(
            partial(CompoundControlWidget.change_type_index, widget))

        # Add a parameter to tell us if the widget is optional
        widget.optional = trait.optional

        # Create the label associated with the enum widget
        control_label = trait.label
        if control_label is None:
            control_label = control_name
        if label_class is None:
            label_class = Qt.QLabel
        if control_label is not None:
            label = (label_class(control_label, parent), lwidget)
        else:
            label = lwidget

        return (widget, label)

    @staticmethod
    def as_ctrait(thandler):
        if hasattr(thandler, 'as_ctrait'):
            return thandler.as_ctrait()

        if hasattr(thandler, 'aType'):
            ttype = thandler.aType
        elif hasattr(thandler, 'aClass'):
            ttype = thandler.aClass
        else:
            ttype = thandler

        if isinstance(ttype, type):
            if ttype.__name__ == 'str':
                ttype = traits.Str()
            elif ttype.__name__ == 'bytes':
                ttype = traits.Bytes()
            elif ttype.__name__ == 'unicode':
                ttype = traits.Unicode()
            #else:
                #ttype = ttype()

        if hasattr(ttype, 'as_ctrait'):
            ttype = ttype.as_ctrait()

        return ttype

    @staticmethod
    def create_compound_widget(widget):
        control_widget = widget.parent()
        while control_widget \
                and not hasattr(control_widget, 'get_control_class') \
                and not hasattr(control_widget, 'controller_widget'):
            control_widget = control_widget.parent()
        if hasattr(control_widget, 'controller_widget'):
            control_widget = control_widget.controller_widget

        if widget.compound_widget is not None:
            # disconnect it
            try:
                widget.compound_class.disconnect(
                    control_widget, widget.trait_name, widget.compound_widget)
            except Exception:
                pass  # probably something already deleted
            del widget.compound_class

            lay_item = widget.layout().takeAt(0)
            w = lay_item.widget()
            w.deleteLater()
            del w
            del lay_item

            if widget.compound_label:
                if isinstance(widget.compound_label, (tuple, list)):
                    for l in widget.compound_label:
                        l.deleteLater()
                else:
                    widget.compound_label.deleteLater()
        widget.compound_label = None

        trait = widget.trait
        thandler = trait.handler.handlers[widget.current_type_id]
        ttype = CompoundControlWidget.as_ctrait(thandler)
        # Create the control instance and associated label
        control_class = control_widget.get_control_class(ttype)

        control_instance, control_label = control_class.create_widget(
            control_widget, widget.trait_name,
            getattr(control_widget.controller, widget.trait_name),
            ttype, user_data=widget.user_data)
        if control_label:
            if isinstance(control_label, (tuple, list)):
                if len(control_label) != 0:
                    control_label[0].deleteLater() # del only label
                    control_label = control_label[1:]
                    layout = widget.header_widget.layout()
                    for l in control_label:
                        layout.addWidget(l)
                    if control_label:
                        widget.compound_label = control_label
            else:
                control_label.deleteLater()
            del control_label

        widget.compound_widget = control_instance
        widget.compound_class = control_class
        widget.layout().addWidget(control_instance)

        control_class.is_valid(control_instance)
        control_class.update_controller_widget(
            control_widget, widget.trait_name, control_instance)
        control_class.connect(control_widget, widget.trait_name,
                              control_instance)

    @staticmethod
    def change_type_index(widget, index):
        if index != widget.current_type_id:
            widget.current_type_id = index
            CompoundControlWidget.create_compound_widget(widget)

    @staticmethod
    def update_controller(controller_widget, control_name,
                          control_instance, *args, **kwargs):
        """ Update one element of the controller.

        At the end the controller trait value with the name 'control_name'
        will match the controller widget user parameters defined in
        'control_instance'.

        Parameters
        ----------
        controller_widget: ControllerWidget (mandatory)
            a controller widget that contains the controller we want to update
        control_name: str(mandatory)
            the name of the controller widget control we want to synchronize
            with the controller
        control_instance: StrControlWidget (mandatory)
            the instance of the controller widget control we want to
            synchronize with the controller
        """
        # nothing to do here since we delegate to another control widget.
        pass

    @staticmethod
    def update_controller_widget(controller_widget, control_name,
                                 control_instance):
        """ Update one element of the controller widget.

        At the end the controller widget user editable parameter with the
        name 'control_name' will match the controller trait value with the same
        name.

        Parameters
        ----------
        controller_widget: ControllerWidget (mandatory)
            a controller widget that contains the controller we want to update
        control_name: str(mandatory)
            the name of the controller widget control we want to synchronize
            with the controller
        control_instance: CompoundControlWidget (mandatory)
            the instance of the controller widget control we want to
            synchronize with the controller
        """
        try:
            trait_types = control_instance.trait.handler.handlers
        except ReferenceError:
            # widget deleted in the meantime
            return

        if sip.isdeleted(control_instance.__init__.__self__):
            CompoundControlWidget.disconnect(controller_widget, control_name,
                                             control_instance)
            return

        if not hasattr(controller_widget.controller, control_name):
            return  # probably deleting this item
        # Get the controller trait value
        new_controller_value = getattr(
            controller_widget.controller, control_name, None)

        # if the value type has changed, select the appropriate type in
        # compound
        type_id = CompoundControlWidget.type_id_for(trait_types,
                                                    new_controller_value)
        if type_id != control_instance.current_type_id:
            control_instance.type_combo.setCurrentIndex(type_id)

        logger.debug("'CompoundControlWidget' has been updated with value "
                     "'{0}'.".format(new_controller_value))

    @staticmethod
    def type_id_for(handlers, value):
        i = 0
        for i, trait in enumerate(handlers):
            # create a custom object with same traits
            temp = traits.HasTraits()
            ctrait = CompoundControlWidget.as_ctrait(trait)
            tname = 'param'
            temp.add_trait(tname, ctrait)
            try:
                setattr(temp, tname, value)
                # OK we have found a working one
                return i
            except Exception as e:
                pass
        else:
            # should not happen
            print('problem in type_id_for:', handlers, value)
            return 0

    @classmethod
    def connect(cls, controller_widget, control_name, control_instance):
        """ Connect an 'Enum' controller trait and an 'EnumControlWidget'
        controller widget control.

        Parameters
        ----------
        cls: EnumControlWidget (mandatory)
            an EnumControlWidget control
        controller_widget: ControllerWidget (mandatory)
            a controller widget that contains the controller we want to update
        control_name: str (mandatory)
            the name of the controller widget control we want to synchronize
            with the controller
        control_instance: QComboBox (mandatory)
            the instance of the controller widget control we want to
            synchronize with the controller
        """
        # Update one element of the controller widget.
        # Hook: function that will be called to update the specific widget
        # when a trait event is detected.
        controller_hook = SomaPartial(
            cls.update_controller_widget, weak_proxy(controller_widget),
            control_name, weak_proxy(control_instance))

        # When the 'control_name' controller trait value is modified, update
        # the corresponding control
        controller_widget.controller.on_trait_change(
            controller_hook, name=control_name, dispatch='ui')

        # Store the trait - control connection we just build
        control_instance._controller_connections = (controller_hook, )
        logger.debug("Add 'Compound' connection: {0} / {1}".format(
            control_name, control_instance))

    @staticmethod
    def disconnect(controller_widget, control_name, control_instance):
        """ Disconnect an 'Enum' controller trait and an 'EnumControlWidget'
        controller widget control.

        Parameters
        ----------
        controller_widget: ControllerWidget (mandatory)
            a controller widget that contains the controller we want to update
        control_name: str(mandatory)
            the name of the controller widget control we want to synchronize
            with the controller
        control_instance: QComboBox (mandatory)
            the instance of the controller widget control we want to
            synchronize with the controller
        """
        # Get the stored widget and controller hooks
        if not hasattr(control_instance, '_controller_connections'):
            return
        controller_hook = control_instance._controller_connections[0]

        # Remove the controller hook from the 'control_name' trait
        controller_widget.controller.on_trait_change(
            controller_hook, name=control_name, remove=True)

        # Delete the trait - control connection we just remove
        del control_instance._controller_connections



# -*- coding: utf-8 -*-
#
# Soma-base - Copyright (C) CEA, 2013
# Distributed under the terms of the CeCILL-B license, as published by
# the CEA-CNRS-INRIA. Refer to the LICENSE file or to
# http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html
# for details.
#

'''Importation of this module activate a
`virtualenv <https://virtualenv.pypa.io/>`_ if
``os.environ['SOMA_VIRTUALENV']`` contains the directory path of the
*virutalenv*.

This module makes it possible to use a specific *virtualenv* directory in
contexts where it is difficult activate it (for instance in crontab). For
instance, a script importing cubicweb the following way::

    from soma import activate_virtualenv
    import cubicweb

can be set to use a specific *virutalenv* install of *cubicweb*:

.. code-block:: bash

    env SOMA_VIRTUALENV=/path/to/virutalenv python /path/to/script.py
'''

from __future__ import absolute_import
import soma.importer
import os

venv = os.environ.get('SOMA_VIRTUALENV')
if venv:
    activate = os.path.join(venv, 'bin', 'activate_this.py')
    if os.path.exists(activate):
        # This is the way to activate a virtualenv from Python
        soma.importer.execfile(activate, dict(__file__=activate))
    del activate



# -*- coding: utf-8 -*-

from __future__ import print_function

from __future__ import absolute_import
import unittest
import shutil
import os
import tempfile
from soma import uuid
import pickle



class TestUUID(unittest.TestCase):

    def test_uuid(self):
        u1 = uuid.Uuid()
        u2 = uuid.Uuid()
        self.assertNotEqual(u1, u2)
        self.assertNotEqual(str(u1), str(u2))
        self.assertEqual(len(str(u1)), 36)
        self.assertEqual(len(repr(u1)), 38)
        self.assertEqual(u1, uuid.Uuid(str(u1)))
        self.assertIs(u1, uuid.Uuid(u1))
        self.assertEqual(u1, str(u1))
        self.assertTrue(u1 != str(u2))
        self.assertTrue(u1 != 'bloblo')
        self.assertTrue(u1 != 12)
        self.assertTrue(not(u1 == 'bloblo'))
        self.assertTrue(not(u1 == 12))
        with self.assertRaises(ValueError):
            uuid.Uuid('blablah0-bouh-bidi-bada-popogugurbav')
        p = pickle.dumps(u1)
        self.assertEqual(u1, pickle.loads(p))
        p = pickle.dumps(u1, 2)  # test Pickle protocol version 2
        self.assertEqual(u1, pickle.loads(p))
        u3 = uuid.Uuid(b'1cab3907-9056-4694-a1d5-266ed5b6ebe3')
        u4 = uuid.Uuid(u'1cab3907-9056-4694-a1d5-266ed5b6ebe3')
        self.assertEqual(u3, u4)


def test():
    suite = unittest.TestLoader().loadTestsFromTestCase(TestUUID)
    runtime = unittest.TextTestRunner(verbosity=2).run(suite)
    return runtime.wasSuccessful()


if __name__ == "__main__":
    test()



# -*- coding: utf-8 -*-
#
# SOMA - Copyright (C) CEA, 2015
# Distributed under the terms of the CeCILL-B license, as published by
# the CEA-CNRS-INRIA. Refer to the LICENSE file or to
# http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html
# for details.
#

# System import
from __future__ import absolute_import
import logging
import sys
import os
import six

# Soma import
from soma.qt_gui.qt_backend import QtGui, QtCore
from soma.functiontools import partial
from soma.qt_gui.controller_widget import ControllerWidget
from soma.qt_gui.controller_widget import weak_proxy
import sip

# Qt import
try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    _fromUtf8 = lambda s: s

QtCore.QResource.registerResource(os.path.join(os.path.dirname(
    os.path.dirname(__file__)), 'resources', 'widgets_icons.rcc'))

# Define the logger
logger = logging.getLogger(__name__)


class ControllerControlWidget(object):

    """ Control to enter an instance of controller.
    """

    #
    # Public members
    #

    @staticmethod
    def is_valid(control_instance, *args, **kwargs):
        """ Method to check if the new control values are correct.

        If the new controller controls values are not correct, the backroung
        color of each control in the controller will be red.

        Parameters
        ----------
        control_instance: QFrame (mandatory)
            the control widget we want to validate

        Returns
        -------
        valid: bool
            True if the control values are valid,
            False otherwise
        """
        # Initilaized the output
        valid = True

        # Go through all the controller widget controls
        controller_widget = control_instance.controller_widget
        for control_name, control_groups \
                in six.iteritems(controller_widget._controls):

            if not control_groups:
                continue
            # Unpack the control item
            trait, control_class, control_instance, control_label \
                = next(iter(control_groups.values()))

            # Call the current control specific check method
            valid = control_class.is_valid(control_instance)

            # Stop checking if a wrong control has been found
            if not valid:
                break

        return valid

    @classmethod
    def check(cls, control_instance):
        """ Check if a controller widget controller control is filled correctly.

        Parameters
        ----------
        cls: ControllerControlWidget (mandatory)
            a ControllerControlWidget control
        control_instance: QFrame (mandatory)
            the control widget we want to validate
        """
        pass

    @staticmethod
    def add_callback(callback, control_instance):
        """ Method to add a callback to the control instance when the controller
        trait is modified

        Parameters
        ----------
        callback: @function (mandatory)
            the function that will be called when a 'textChanged' signal is
            emited.
        control_instance: QFrame (mandatory)
            the control widget we want to validate
        """
        pass

    @staticmethod
    def create_widget(parent, control_name, control_value, trait,
                      label_class=None, user_data=None):
        """ Method to create the controller widget.

        Parameters
        ----------
        parent: QWidget (mandatory)
            the parent widget
        control_name: str (mandatory)
            the name of the control we want to create
        control_value: instance of Controller (mandatory)
            the default control value
        trait: Tait (mandatory)
            the trait associated to the control
        label_class: Qt widget class (optional, default: None)
            the label widget will be an instance of this class. Its constructor
            will be called using 2 arguments: the label string and the parent
            widget.

        Returns
        -------
        out: 2-uplet
            a two element tuple of the form (control widget: ,
            associated labels: (a label QLabel, the tools QWidget))
        """
        # Create the list widget: a frame
        frame = QtGui.QFrame(parent=parent)
        frame.setFrameShape(QtGui.QFrame.StyledPanel)

        # Create tools to interact with the list widget: expand or collapse -
        # add a list item - remove a list item
        tool_widget = QtGui.QWidget(parent)
        layout = QtGui.QHBoxLayout()
        layout.addStretch(1)
        layout.setContentsMargins(0, 0, 0, 0)
        layout.setSpacing(2)
        tool_widget.setLayout(layout)
        # Create the tool buttons
        resize_button = QtGui.QToolButton()
        layout.addWidget(resize_button)
        # Set the tool icons
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(
            _fromUtf8(":/soma_widgets_icons/nav_down")),
            QtGui.QIcon.Normal, QtGui.QIcon.Off)
        resize_button.setIcon(icon)
        resize_button.setFixedSize(30, 22)
        frame.user_data = user_data

        editable_labels = False
        handler = getattr(trait, 'handler', trait)
        if handler.inner_traits():
            editable_labels = True
            frame.inner_trait = trait.handler.inner_traits()[0]

            add_button = QtGui.QToolButton()
            delete_button = QtGui.QToolButton()
            layout.addWidget(add_button)
            # Set the tool icons
            icon = QtGui.QIcon()
            icon.addPixmap(
                QtGui.QPixmap(_fromUtf8(":/soma_widgets_icons/add")),
                QtGui.QIcon.Normal, QtGui.QIcon.Off)
            add_button.setIcon(icon)
            add_button.setFixedSize(30, 22)
            delete_button.setFixedSize(30, 22)
            # Add list item callback
            add_hook = partial(
                ControllerControlWidget.add_item, weak_proxy(parent),
                control_name, weak_proxy(frame))
            add_button.clicked.connect(add_hook)

        # Create the associated controller widget
        controller_widget = ControllerWidget(control_value, parent=frame,
                                             live=True,
                                             editable_labels=editable_labels,
                                             user_data=user_data)

        # Store some parameters in the list widget
        frame.trait = trait
        frame.controller = control_value
        frame.controller_widget = controller_widget
        frame.connected = False

        # Add the list controller widget to the list widget
        frame.setLayout(controller_widget.layout())

        # Set some callback on the controller control tools
        # Resize callback
        resize_hook = partial(
            ControllerControlWidget.expand_or_collapse, weak_proxy(frame),
            weak_proxy(resize_button))
        resize_button.clicked.connect(resize_hook)

        if getattr(trait, 'expanded') is False:
            ControllerControlWidget.set_expanded(frame, resize_button, False)

        # Create the label associated with the controller widget
        control_label = trait.label
        if control_label is None:
            control_label = control_name
        if label_class is None:
            label_class = QtGui.QLabel
        if control_label is not None:
            label = label_class(control_label, parent)
        else:
            label = None

        return (frame, (label, tool_widget))

    @staticmethod
    def update_controller(controller_widget, control_name, control_instance,
                          *args, **kwarg):
        """ Update one element of the controller.

        At the end the controller trait value with the name 'control_name'
        will match the controller widget user parameters defined in
        'control_instance'.

        Parameters
        ----------
        controller_widget: ControllerWidget (mandatory)
            a controller widget that contains the controller we want to update
        control_name: str(mandatory)
            the name of the controller widget control we want to synchronize
            with the controller
        control_instance: QFrame (mandatory)
            the instance of the controller widget control we want to
            synchronize with the controller
        """
        control_instance.controller_widget.update_controller()

    @classmethod
    def update_controller_widget(cls, controller_widget, control_name,
                                 control_instance):
        """ Update one element of the controller control widget.

        At the end the controller control widget user editable parameter with the
        name 'control_name' will match the controller trait value with the same
        name.

        Parameters
        ----------
        controller_widget: ControllerWidget (mandatory)
            a controller widget that contains the controller we want to update
        control_name: str(mandatory)
            the name of the controller widget control we want to synchronize
            with the controller
        control_instance: QFrame (mandatory)
            the instance of the controller widget control we want to
            synchronize with the controller
        """
        try:
            test = control_instance.controller_widget
        except ReferenceError:
            # widget deleted in the meantime
            return

        if sip.isdeleted(control_instance.__init__.__self__):
            ControllerControlWidget.disconnect(controller_widget, control_name,
                                               control_instance)
            return

        # One callback has not been removed properly
        control_instance.controller_widget.update_controller_widget()

    @classmethod
    def connect(cls, controller_widget, control_name, control_instance):
        """ Connect a 'Controller' controller trait and a 'ControllerControlWidget'
        controller widget control.

        Parameters
        ----------
        cls: StrControlWidget (mandatory)
            a StrControlWidget control
        controller_widget: ControllerWidget (mandatory)
            a controller widget that contains the controller we want to update
        control_name: str (mandatory)
            the name of the controller widget control we want to synchronize
            with the controller
        control_instance: QFrame (mandatory)
            the instance of the controller widget control we want to
            synchronize with the controller
        """
        # Check if the control is connected
        if not control_instance.connected:
            control_instance.controller_widget.connect()
            # Update the list control connection status
            control_instance.connected = True

    @staticmethod
    def disconnect(controller_widget, control_name, control_instance):
        """ Disconnect a 'Controller' controller trait and a 'ControllerControlWidget'
        controller widget control.

        Parameters
        ----------
        cls: StrControlWidget (mandatory)
            a StrControlWidget control
        controller_widget: ControllerWidget (mandatory)
            a controller widget that contains the controller we want to update
        control_name: str (mandatory)
            the name of the controller widget control we want to synchronize
            with the controller
        control_instance: QFrame (mandatory)
            the instance of the controller widget control we want to
            synchronize with the controller
        """
        # Check if the control is connected
        if control_instance.connected:
            control_instance.controller_widget.disconnect()
            # Update the list control connection status
            control_instance.connected = False

    @staticmethod
    def expand_or_collapse(control_instance, resize_button):
        """ Callback to expand or collapse a 'ControllerControlWidget'.

        Parameters
        ----------
        control_instance: QFrame (mandatory)
            the list widget item
        resize_button: QToolButton
            the signal sender
        """
        # Hide the control
        if control_instance.isVisible():
            state = False
        # Show the control
        else:
            state = True
        ControllerControlWidget.set_expanded(control_instance, resize_button,
                                             state)

    @staticmethod
    def set_expanded(control_instance, resize_button, state):
        """ Expand or collapse a 'ControllerControlWidget'.

        Parameters
        ----------
        control_instance: QFrame (mandatory)
            the list widget item
        resize_button: QToolButton
            the signal sender
        state: bool
            expanded (True) or collapsed (False)
        """
        # Change the icon depending on the button status
        icon = QtGui.QIcon()

        # Hide the control
        if not state:
            control_instance.hide()
            icon.addPixmap(
                QtGui.QPixmap(_fromUtf8(":/soma_widgets_icons/nav_right")),
                QtGui.QIcon.Normal, QtGui.QIcon.Off)

        # Show the control
        else:
            control_instance.show()
            icon.addPixmap(
                QtGui.QPixmap(_fromUtf8(":/soma_widgets_icons/nav_down")),
                QtGui.QIcon.Normal, QtGui.QIcon.Off)

        # Set the new button icon
        resize_button.setIcon(icon)

    #
    # Callbacks
    #

    @staticmethod
    def add_item(controller_widget, control_name, control_instance):
        """ Append one element in the controller widget.

        Parameters
        ----------
        controller_widget: ControllerWidget (mandatory)
            a controller widget that contains the controller we want to update
        control_name: str(mandatory)
            the name of the controller widget control we want to synchronize
            with the controller
        control_instance: QFrame (mandatory)
            the instance of the controller widget control we want to
            synchronize with the controller
        """
        # Get a new key name
        trait_name = 'new_item'
        i = 1
        while control_instance.controller.trait(trait_name):
            trait_name = 'new_item_%d' % i
            i += 1

        # Add the new trait to the inner list controller
        control_instance.controller.add_trait(
            trait_name, control_instance.inner_trait)

        # update interface
        control_instance.controller_widget.update_controls()
        # update the real underlying dict object
        control_instance.controller_widget.update_controller()

        logger.debug("Add 'ControllerControlWidget' '{0}' new trait "
                     "callback.".format(trait_name))



# -*- coding: utf-8 -*-
#
# SOMA - Copyright (C) CEA, 2015
# Distributed under the terms of the CeCILL-B license, as published by
# the CEA-CNRS-INRIA. Refer to the LICENSE file or to
# http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html
# for details.
#

# System import
from __future__ import absolute_import
import logging
from functools import partial

# Define the logger
logger = logging.getLogger(__name__)

# Soma import
from soma.qt_gui.qt_backend import QtGui, Qt
from soma.utils.functiontools import SomaPartial
from soma.qt_gui.controller_widget import weak_proxy
import sip

import traits.api as traits


class BoolControlWidget(object):

    """ Control to set or unset an option.
    """

    @staticmethod
    def is_valid(control_instance, *args, **kwargs):
        """ Method to check if the new control value is correct.

        Parameters
        ----------
        control_instance: QCheckBox (mandatory)
            the control widget we want to validate

        Returns
        -------
        out: bool
            always True since the control value is always valid
        """
        return True

    @classmethod
    def check(cls, control_instance):
        """ Check if a controller widget control is filled correctly.

        Parameters
        ----------
        cls: BoolControlWidget (mandatory)
            a BoolControlWidget control
        control_instance: QCheckBox (mandatory)
            the control widget we want to validate
        """
        # Hook: function that will be called to check for typo
        # when a 'clicked' qt signal is emited
        widget_callback = partial(cls.is_valid, weak_proxy(control_instance))

        # Execute manually the first time the control check method
        widget_callback()

        # When a qt 'clicked' signal is emited, check if the new
        # user value is correct
        control_instance.clicked.connect(widget_callback)

    @staticmethod
    def add_callback(callback, control_instance):
        """ Method to add a callback to the control instance when a 'clicked'
        signal is emited.

        Parameters
        ----------
        callback: @function (mandatory)
            the function that will be called when a 'clicked' signal is emited.
        control_instance: QCheckBox (mandatory)
            the control widget we want to validate
        """
        control_instance.clicked.connect(callback)

    @staticmethod
    def create_widget(parent, control_name, control_value, trait,
                      label_class=None, user_data=None):
        """ Method to create the bool widget.

        Parameters
        ----------
        parent: QWidget (mandatory)
            the parent widget
        control_name: str (mandatory)
            the name of the control we want to create
        control_value: str (mandatory)
            the default control value
        trait: Tait (mandatory)
            the trait associated to the control
        label_class: Qt widget class (optional, default: None)
            the label widget will be an instance of this class. Its constructor
            will be called using 2 arguments: the label string and the parent
            widget.

        Returns
        -------
        out: 2-uplet
            a two element tuple of the form (control widget: QLineEdit,
            associated label: QLabel)
        """
        # Create the widget that will be used to select an option
        widget = QtGui.QCheckBox(parent)

        # Add a widget parameter to tell us if the widget is already connected
        widget.connected = False

        # Create the label associated with the bool widget
        control_label = trait.label
        if control_label is None:
            control_label = control_name
        if label_class is None:
            label_class = QtGui.QLabel
        if control_label is not None:
            label = label_class(control_label, parent)
        else:
            label = None

        return (widget, label)

    @staticmethod
    def update_controller(controller_widget, control_name, control_instance,
                          *args, **kwargs):
        """ Update one element of the controller.

        At the end the controller trait value with the name 'control_name'
        will match the controller widget user parameters defined in
        'control_instance'.

        Parameters
        ----------
        controller_widget: ControllerWidget (mandatory)
            a controller widget that contains the controller we want to update
        control_name: str(mandatory)
            the name of the controller widget control we want to synchronize
            with the controller
        control_instance: QCheckBox (mandatory)
            the instance of the controller widget control we want to
            synchronize with the controller
        """
        # Update the controller only if the control is valid
        if BoolControlWidget.is_valid(control_instance):

            # Get the control value
            new_trait_value = bool(control_instance.isChecked())

            # value is manually modified: protect it
            if getattr(controller_widget.controller, control_name) \
                    != new_trait_value:
                controller_widget.controller.protect_parameter(control_name)
            # Set the control value to the controller associated trait
            setattr(controller_widget.controller, control_name,
                    new_trait_value)
            logger.debug(
                "'BoolControlWidget' associated controller trait '{0}' has "
                "been updated with value '{1}'.".format(
                    control_name, new_trait_value))

    @staticmethod
    def update_controller_widget(controller_widget, control_name,
                                 control_instance):
        """ Update one element of the controller widget.

        At the end the controller widget user editable parameter with the
        name 'control_name' will match the controller trait value with the same
        name.

        Parameters
        ----------
        controller_widget: ControllerWidget (mandatory)
            a controller widget that contains the controller we want to update
        control_name: str(mandatory)
            the name of the controller widget control we want to synchronize
            with the controller
        control_instance: QCheckBox (mandatory)
            the instance of the controller widget control we want to
            synchronize with the controller
        """
        try:
            test = control_instance.setTristate
        except ReferenceError:
            # widget deleted in the meantime
            return

        if sip.isdeleted(control_instance.__init__.__self__):
            BoolControlWidget.disconnect(controller_widget, control_name,
                                         control_instance)
            return

        # Get the trait value
        new_controller_value = getattr(
            controller_widget.controller, control_name, False)

        if new_controller_value is traits.Undefined:
            control_instance.setTristate(True)
        # Set the trait value to the bool control
        if new_controller_value is True:
            new_controller_checked = Qt.Qt.Checked
        elif new_controller_value is False:
            new_controller_checked = Qt.Qt.Unchecked
        else:
            new_controller_checked = Qt.Qt.PartiallyChecked
        control_instance.setCheckState(new_controller_checked)
        logger.debug("'BoolControlWidget' has been updated with value "
                     "'{0}'.".format(new_controller_value))

    @classmethod
    def connect(cls, controller_widget, control_name, control_instance):
        """ Connect a 'Bool' controller trait and a 'BoolControlWidget'
        controller widget control.

        Parameters
        ----------
        cls: BoolControlWidget (mandatory)
            a BoolControlWidget control
        controller_widget: ControllerWidget (mandatory)
            a controller widget that contains the controller we want to update
        control_name: str (mandatory)
            the name of the controller widget control we want to synchronize
            with the controller
        control_instance: QCheckBox (mandatory)
            the instance of the controller widget control we want to
            synchronize with the controller
        """
        # Check if the control is connected
        if not control_instance.connected:

            # Update one element of the controller.
            # Hook: function that will be called to update a specific
            # controller trait when a 'textChanged' qt signal is emited
            widget_hook = partial(cls.update_controller,
                                  weak_proxy(controller_widget),
                                  control_name, weak_proxy(control_instance))

            # When a qt 'clicked' signal is emited, update the
            # 'control_name' controller trait value
            control_instance.clicked.connect(widget_hook)

            # Update the control.
            # Hook: function that will be called to update the control value
            # when the 'control_name' controller trait is modified.
            controller_hook = SomaPartial(
                cls.update_controller_widget, weak_proxy(controller_widget),
                control_name, weak_proxy(control_instance))

            # When the 'control_name' controller trait value is modified,
            # update the corresponding control
            controller_widget.controller.on_trait_change(
                controller_hook, name=control_name, dispatch='ui')

            # Store the trait - control connection we just build
            control_instance._controller_connections = (
                widget_hook, controller_hook)
            logger.debug("Add 'String' connection: {0}.".format(
                control_instance._controller_connections))

            # Update the control connection status
            control_instance.connected = True

    @staticmethod
    def disconnect(controller_widget, control_name, control_instance):
        """ Disconnect a 'Bool' controller trait and a 'BoolControlWidget'
        controller widget control.

        Parameters
        ----------
        controller_widget: ControllerWidget (mandatory)
            a controller widget that contains the controller we want to update
        control_name: str(mandatory)
            the name of the controller widget control we want to synchronize
            with the controller
        control_instance: QCheckBox (mandatory)
            the instance of the controller widget control we want to
            synchronize with the controller
        """
        # Check if the control is connected
        if control_instance.connected:

            # Get the stored widget and controller hooks
            (widget_hook,
             controller_hook) = control_instance._controller_connections

            # Remove the controller hook from the 'control_name' trait
            controller_widget.controller.on_trait_change(
                controller_hook, name=control_name, remove=True)

            # Remove the widget hook associated with the qt 'clicked' signal
            if not sip.isdeleted(control_instance.__init__.__self__):
                control_instance.clicked.disconnect(widget_hook)

            # Delete the trait - control connection we just remove
            del control_instance._controller_connections

            # Update the control connection status
            control_instance.connected = False



# -*- coding: utf-8 -*-

#  This software and supporting documentation are distributed by
#      Institut Federatif de Recherche 49
#      CEA/NeuroSpin, Batiment 145,
#      91191 Gif-sur-Yvette cedex
#      France
#
# This software is governed by the CeCILL-B license under
# French law and abiding by the rules of distribution of free software.
# You can  use, modify and/or redistribute the software under the
# terms of the CeCILL-B license as circulated by CEA, CNRS
# and INRIA at the following URL "http://www.cecill.info".
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL-B license and that you accept its terms.

'''
Universal unique identifier.

- author: Yann Cointepas
- organization: NeuroSpin
- license: `CeCILL B <http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html>`_
'''
from __future__ import absolute_import
import six
__docformat__ = "epytext en"

import struct
import random
import binascii
import sys

#-------------------------------------------------------------------------


class Uuid(object):

    '''
    An Uuid instance is a universal unique identifier. It is a 128 bits
    random value.
    '''
    def __new__(cls, value=None):
        if isinstance(value, Uuid):
            return value
        return object.__new__(cls)

    def __init__(self, uuid=None):
        '''
        Uuid constructor. If *uuid* is ommited or *None*, a new random
        Uuid is created; if it is a string if must be 36 characters long and
        follow the pattern::

            XXXXXXXX-XXXX-XXXX-XXXX-XXXXXXXXXXXX

        where ``X`` is an
        hexadecimal digit (example:
        ``'ad2d8fb0-7831-50bc-2fb6-5df048304001'``).

        If *uuid* is an Uuid instance, no new instance is created, in this
        case, *Uuid(uuid)* returns *uuid*.
        '''
        if isinstance(uuid, Uuid):
            return
        if uuid is None:
            # Generate a new 128 bits uuid
            self.__uuid = struct.pack('QQ', random.randrange(2 ** 64 - 1),
                                      random.randrange(2 ** 64 - 1))
        else:
            try:
                uuid = six.ensure_binary(uuid, encoding='ascii')
                self.__uuid = binascii.unhexlify(uuid[0:8] + uuid[9:13] +
                                                 uuid[14:18] + uuid[19:23] +
                                                 uuid[24:36])
            except Exception:
                raise ValueError("Invalid uuid string %s" % (repr(uuid), ))

    def __getnewargs__(self):
        return (str(self), )

    def __str__(self):
        if not isinstance(self.__uuid, bytes):
            # this should not happen, but has been seen in some places
            import warnings
            warnings.warn('soma.uuid.Uuid: self.__uuid is not of type bytes, '
                          'but {0}. This is not supposed to happen.'
                          .format(type(self.__uuid)))
            self.__uuid = bytes(self.__uuid, encoding='utf-8')
        return six.ensure_str(
            binascii.hexlify(self.__uuid[0:4]) + b'-' +
            binascii.hexlify(self.__uuid[4:6]) + b'-' +
            binascii.hexlify(self.__uuid[6:8]) + b'-' +
            binascii.hexlify(self.__uuid[8:10]) + b'-' +
            binascii.hexlify(self.__uuid[10:16])
        )

    def __repr__(self):
        return repr(str(self))

    def __hash__(self):
        return hash(self.__uuid)

    def __eq__(self, other):
        if isinstance(other, Uuid):
            return self.__uuid == other.__uuid
        elif isinstance(other, six.string_types):  # assume string-like object (str or unicode)
            try:
                uuid_other = Uuid(other)
            except ValueError:
                return False
            return self.__uuid == uuid_other.__uuid
        else:
            return False

    def __ne__(self, other):
        if isinstance(other, Uuid):
            return self.__uuid != other.__uuid
        elif isinstance(other, six.string_types):  # assume string-like object (str or unicode)
            try:
                uuid_other = Uuid(other)
            except ValueError:
                return True
            return self.__uuid != uuid_other.__uuid
        else:
            return True



# -*- coding: utf-8 -*-

#  This software and supporting documentation are distributed by
#      Institut Federatif de Recherche 49
#      CEA/NeuroSpin, Batiment 145,
#      91191 Gif-sur-Yvette cedex
#      France
#
# This software is governed by the CeCILL-B license under
# French law and abiding by the rules of distribution of free software.
# You can  use, modify and/or redistribute the software under the
# terms of the CeCILL-B license as circulated by CEA, CNRS
# and INRIA at the following URL "http://www.cecill.info".
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL-B license and that you accept its terms.

'''
Sorted dictionary behave like a dictionary but keep the item insertion
order.

* author: Yann Cointepas
* organization: NeuroSpin
* license: CeCILL B (http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html)

In addition OrderedDict is provided here, either as the standard
collections.OrderedDict class if python version >= 2.7, or based on
SortedDictionary if python version < 2.7.
'''
from __future__ import print_function

from __future__ import absolute_import
__docformat__ = "restructuredtext en"

from collections import OrderedDict
import sys
import six
import inspect
from soma.undefined import Undefined

from six.moves import collections_abc


class SortedDictionary(dict):

    '''
    Sorted dictionary behave like a dictionary but keep the item insertion
    order. In addition to python 2.7 OrderedDict, SortedDictionary also has
    an :py:meth:`insert` method allowing insersion at a specified index
    position.

    Example:

    ::

        from SortedDictionary import SortedDictionary
        sd = SortedDictionary(('fisrt', 1), ('second', 2))
        sd['third'] = 3
        sd.insert(0, 'zero', 0)
        sd.items() == [('zero', 0), ('fisrt', 1), ('second', 2), ('third', 3)]
    '''

    def __init__(self, *args):
        '''
        Initialize the dictionary with a list of (key, value) pairs.
        '''
        super(SortedDictionary, self).__init__()
        self.sortedKeys = []
        if len(args) == 1 and (
                isinstance(args[0], list)
                or inspect.isgenerator(args[0])
                or (isinstance(args[0], collections_abc.ItemsView))):
            elements = args[0]  # dict / OrderedDict compatibility
        else:
            elements = args
        for key, value in elements:
            self[key] = value

    def keys(self):
        '''
        Returns
        -------
        list
            sorted list of keys
        '''
        return self.sortedKeys

    def items(self):
        '''
        Returns
        -------
        list
            sorted list of (key, value) pairs
        '''
        if six.PY2:
            return list(self.iteritems())
        else:
            return self.iteritems()

    def values(self):
        '''
        Returns
        -------
        values: list
            sorted list of values
        '''
        if six.PY2:
            return list(self.itervalues())
        else:
            return self.itervalues()

    def __setitem__(self, key, value):
        if key not in self:
            if 'sortedKeys' not in self.__dict__:
                # this happens during pickle.load() with python3
                self.sortedKeys = []
            self.sortedKeys.append(key)
        super(SortedDictionary, self).__setitem__(key, value)

    def __delitem__(self, key):
        super(SortedDictionary, self).__delitem__(key)
        self.sortedKeys.remove(key)

    def __getstate__(self):
        return list(self.items())

    def __setstate__(self, state):
        SortedDictionary.__init__(self, *state)

    def __iter__(self):
        '''
        returns an iterator over the sorted keys
        '''
        return iter(self.sortedKeys)

    def iterkeys(self):
        '''
        returns an iterator over the sorted keys
        '''
        return iter(self.sortedKeys)

    def itervalues(self):
        '''
        returns an iterator over the sorted values
        '''
        for k in self:
            yield self[k]

    def iteritems(self):
        '''
        returns an iterator over the sorted (key, value) pairs
        '''
        for k in self:
            try:
                yield (k, self[k])
            except KeyError:
                print('!SortedDictionary error!', self.keys(), self.sortedKeys)
                raise

    def insert(self, index, key, value):
        '''
        insert a (key, value) pair in sorted dictionary before position
        ``index``. If ``key`` is already in the dictionary, a KeyError is
        raised.

        Parameters
        ----------
        index: integer
            index of key in the sorted keys
        key: key to insert
            value associated to key
        '''
        if key in self:
            raise KeyError(key)
        self.sortedKeys.insert(index, key)
        super(SortedDictionary, self).__setitem__(key, value)

    def index(self, key):
        """
        Returns the index of the key in the sorted dictionary, or -1 if this key
        isn't in the dictionary.
        """
        try:
            i = self.sortedKeys.index(key)
        except Exception:
            i = -1
        return i

    def clear(self):
        '''
        Remove all items from dictionary
        '''
        del self.sortedKeys[:]
        super(SortedDictionary, self).clear()

    def sort(self, key=None, reverse=False):
        """Sorts the dictionary using key function key.

        Parameters
        ----------
        key: function key
        """
        self.sortedKeys.sort(key=key, reverse=reverse)

    def compValues(self, key1, key2):
        """
        Use this comparaison function in sort method parameter in order to sort
        the dictionary by values.
        if data[key1]<data[key2] return -1
        if data[key1]>data[key2] return 1
        if data[key1]==data[key2] return 0
        """
        e1 = self[key1]
        e2 = self[key2]
        if e1 < e2:
            return -1
        elif e1 > e2:
            return 1
        return 0

    def setdefault(self, key, value=None):
        result = self.get(key, Undefined)
        if result is Undefined:
            self[key] = value
            result = value
        return result

    def pop(self, key, default=Undefined):
        if default is Undefined:
            result = super(SortedDictionary, self).pop(key)
        else:
            result = super(SortedDictionary, self).pop(key, Undefined)
            if result is Undefined:
                return default
        self.sortedKeys.remove(key)
        return result

    def popitem(self):
        result = super(SortedDictionary, self).popitem()
        try:
            self.sortedKeys.remove(result[0])
        except ValueError:
            pass
        return result

    def __repr__(self):
        return '{' + ', '.join(repr(k) + ': ' + repr(v)
                               for k, v in self.iteritems()) + '}'

    def update(self, dict_obj):
        for k, v in six.iteritems(dict_obj):
            self[k] = v

    def copy(self):
        copied = self.__class__()
        copied.update(self)
        return copied



# -*- coding: utf-8 -*-

#  This software and supporting documentation are distributed by
#      Institut Federatif de Recherche 49
#      CEA/NeuroSpin, Batiment 145,
#      91191 Gif-sur-Yvette cedex
#      France
#
# This software is governed by the CeCILL license version 2 under
# French law and abiding by the rules of distribution of free software.
# You can  use, modify and/or redistribute the software under the
# terms of the CeCILL license version 2 as circulated by CEA, CNRS
# and INRIA at the following URL "http://www.cecill.info".
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license version 2 and that you accept its terms.

'''
QLineEditModificationTimer and TimeredQLineEdit classes associate a
QtCore.QTimer to a QtGui.QLineEdit in order to signal user
modification only after an inactivity period.

* author: Yann Cointepas
* organization: NeuroSpin
* license: `CeCILL B <http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html>`_
'''
from __future__ import absolute_import
__docformat__ = "restructuredtext en"

from soma.qt_gui.qt_backend import QtCore, QtGui, Qt
from soma.qt_gui.predef_lineedit import QPredefLineEdit
import weakref

#-------------------------------------------------------------------------


class QLineEditModificationTimer(QtCore.QObject):

    '''
    A QLineEditModificationTimer instance is accociated to a
    QtGui.QLineEdit instance, it listens all user modification (Qt
    signal 'textChanged( const QString & )') and emits a
    signal 'userModification()' when timerInterval milliseconds passed
    since the last user modification.
    '''
    # Default timer interval in milliseconds
    defaultTimerInterval = 2000
    userModification = QtCore.Signal()

    def __init__(self, qLineEdit, timerInterval=None):
        '''
        .. seealso:: :class:`TimeredQLineEdit`

        Parameters
        ----------
        qLineEdit: (QtGui.QLineEdit instance)
            widget associated with this QLineEditModificationTimer.
        timerInterval: (milliseconds)
            minimum inactivity period before emitting
            userModification signal. Default value is
            QLineEditModificationTimer.defaultTimerInterval

        '''

        QtCore.QObject.__init__(self)
        # QLineEdit<qt.QLineEdit> instance associated with this
        # QLineEditModificationTimer
        self.qLineEdit = weakref.proxy(qLineEdit)
        if timerInterval is None:
            self.timerInterval = self.defaultTimerInterval
        else:
            # minimum inactivity period before emitting C{userModification}
            # signal.
            self.timerInterval = timerInterval
        self.__timer = QtCore.QTimer(self)
        self.__timer.setSingleShot(True)
        self.__internalModification = False
        self.qLineEdit.textChanged.connect(self._userModification)
        self.qLineEdit.editingFinished.connect(self._noMoreUserModification)
        self.__timer.timeout.connect(self.modificationTimeout)

    def close(self):
        self.stop()
        self.qLineEdit.textChanged.disconnect(self._userModification)
        # emit a last signal if modifs have been done
        self.qLineEdit.editingFinished.disconnect(self._noMoreUserModification)
        self.__timer.timeout.disconnect(self.modificationTimeout)

    def _userModification(self, value):
        if not self.__internalModification:
            self.__timer.start(self.timerInterval)

    def modificationTimeout(self):
        self.userModification.emit()  # self.qLineEdit.text())

    def _noMoreUserModification(self):
        if self.__timer.isActive():
            self.__timer.stop()
            self.userModification.emit()  # self.qLineEdit.text())

    def stopInternalModification(self):
        '''
        Stop emitting ``userModification`` signal when associated
        :pyqt:`QLineEdit <QtGui/QLineEdit.html>` is modified.

        .. seealso:: :meth:`startInternalModification`
        '''
        self.__internalModification = False

    def startInternalModification(self):
        '''
        Restart emitting C{userModification} signal when associated
        :pyqt:`QLineEdit <QtGui/QLineEdit.html>` is modified.

        .. seealso:: :meth:`stopInternalModification`
        '''
        self.__internalModification = True

    def stop(self):
        '''
        Stop the timer if it is active.
        '''
        self.__timer.stop()

    def isActive(self):
        '''
        Returns True if the timer is active, or False otherwise.
        '''
        return self.__timer.isActive()


#-------------------------------------------------------------------------
class TimeredQLineEdit(QPredefLineEdit):

    '''
    Create a QLineEdit instance that has an private attribute
    containing a QLineEditModificationTimer associated to self. Whenever
    the internal QLineEditModificationTimer emits a userModification
    signal, this signal is also emited by the TimeredQLineEdit instance.
    '''

    userModification = QtCore.Signal()

    def __init__(self, *args, **kwargs):
        '''
        All non keyword parameters of the constructor are passed to
        :pyqt:`QLineEdit <QtGui/QLineEdit.html>` constructor. An optional
        *timerInterval* keyword parameter can be given, it is passed to
        :class:`QLineEditModificationTimer` constructor. At the time this class
        was created, :pyqt:`QLineEdit <QtGui/QLineEdit.html>` constructor did
        not accept keyword parameters.
        '''
        timerInterval = kwargs.pop('timerInterval', None)
        if kwargs:
            super(TimeredQLineEdit, self).__init__(*args, **kwargs)
        else:
            super(TimeredQLineEdit, self).__init__(*args)
        self.__timer = QLineEditModificationTimer(self,
                                                  timerInterval=timerInterval)
        self.__timer.userModification.connect(self.userModification)

    def stopInternalModification(self):
        '''
        .. seealso:: :meth:`QLineEditModificationTimer.stopInternalModification`
        '''
        self.__timer.stopInternalModification()

    def startInternalModification(self):
        '''
        .. seealso:: :meth:`QLineEditModificationTimer.startInternalModification`
        '''
        self.__timer.startInternalModification()

    def close(self):
        self.__timer.close()
        super(TimeredQLineEdit, self).close()

    def set_value(self, value):
        super(TimeredQLineEdit, self).set_value(value)
        self.userModification.emit()



# -*- coding: utf-8 -*-

#  This software and supporting documentation are distributed by
#      Institut Federatif de Recherche 49
#      CEA/NeuroSpin, Batiment 145,
#      91191 Gif-sur-Yvette cedex
#      France
#
# This software is governed by the CeCILL-B license under
# French law and abiding by the rules of distribution of free software.
# You can  use, modify and/or redistribute the software under the
# terms of the CeCILL-B license as circulated by CEA, CNRS
# and INRIA at the following URL "http://www.cecill.info".
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL-B license and that you accept its terms.

'''
:attr:`Undefined` (or :attr:`undefined`) is a constant that can be used as
a special value different from any other Python value including *None*.

If the :traits:`traits <index.html>` library is available, we just use its
:traits:`Undefined <traits_api_reference/trait_base.html#traits.trait_base.Undefined>` implementation.

Example::

    from soma.undefined import undefined

    if object.value is undefined:
        # do something

* author: Yann Cointepas
* organization: NeuroSpin
* license: `CeCILL B <http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html>`_
'''
from __future__ import print_function

from __future__ import absolute_import
__docformat__ = "restructuredtext en"

try:
    # If possible, use _Undefined from traits
    from traits.api import Undefined
    # Undefined is also defined for backward compatibility
    undefined = Undefined
except ImportError:
    undefined = None

if undefined is None:
    # _Undefined cannot be imported from traits, provides an implementation

    from soma.singleton import Singleton

    #-------------------------------------------------------------------------
    class UndefinedClass(Singleton):

        '''
        *UndefinedClass* instance is used to represent an undefined attribute
        value when *None* cannot be used because it can be a valid value.

        Should only be used for value checking.
        '''

        def __repr__(self):
            '''
            Returns
            -------
            ``'<undefined>'``
            '''
            return '<undefined>'

    Undefined = undefined = UndefinedClass()



# -*- coding: utf-8 -*-

from __future__ import print_function

from __future__ import absolute_import
import unittest
import os
import sys
try:
    #from soma import qimage2ndarray
    from soma import qt_gui
    from soma.qt_gui import controller_widget
    from soma.qt_gui import controls
    from soma.qt_gui import generic_table_editor
    from soma.qt_gui import io
    from soma.qt_gui import qtThread
    from soma.qt_gui import tangeSlider
    from soma.qt_gui import timered_widgets

    class TestQtGui(unittest.TestCase):

        def test_qtgui(self):
            from soma.qt_gui import qt_backend
            qt_backend.set_qt_backend(compatible_qt5=True)
            self.assertTrue(qt_backend.get_qt_backend()
                            in ('PyQt4', 'PyQt5', 'PySide'))


    def test():
        suite = unittest.TestLoader().loadTestsFromTestCase(TestQtGui)
        runtime = unittest.TextTestRunner(verbosity=2).run(suite)
        return runtime.wasSuccessful()


    if __name__ == "__main__":
        test()

except ImportError:
    # PyQt not installed
    have_gui = False



# -*- coding: utf-8 -*-
#
# SOMA - Copyright (C) CEA, 2015
# Distributed under the terms of the CeCILL-B license, as published by
# the CEA-CNRS-INRIA. Refer to the LICENSE file or to
# http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html
# for details.
#

# Soma import
from .Str import StrControlWidget
from .Bytes import BytesControlWidget
from .Float import FloatControlWidget
from .Int import IntControlWidget
from .Enum import EnumControlWidget
from .List import ListControlWidget
from .List_offscreen import OffscreenListControlWidget
from .List_File_offscreen import OffscreenListFileControlWidget
from .Bool import BoolControlWidget
from .File import FileControlWidget
from .Directory import DirectoryControlWidget
from .Dict import DictControlWidget
from .Controller import ControllerControlWidget
from .Compound import CompoundControlWidget
from .non_editable import NonEditableControlWidget

import sys

# Define a structure that will contain the mapping between the string trait
# descriptions and the associated control classes
controls = {}


def range_editor(trait):
    t = type(trait.default)
    if t is int:
        return IntControlWidget
    if sys.version_info[0] >= 3:
        long = int  # this is only a trick to fool flake8
    elif t is long:
        return IntControlWidget
    return FloatControlWidget


# Register all control class
controls["Str"] = StrControlWidget
controls["Unicode"] = StrControlWidget
controls["Bytes"] = BytesControlWidget
controls["String"] = StrControlWidget
controls["Any"] = StrControlWidget
controls["Float"] = FloatControlWidget
controls["Int"] = IntControlWidget
controls["Enum"] = EnumControlWidget
controls["Bool"] = BoolControlWidget
controls["File"] = FileControlWidget
controls["Directory"] = DirectoryControlWidget
controls["List"] = ListControlWidget
controls["Instance"] = ControllerControlWidget
controls["TraitInstance"] = ControllerControlWidget
controls["ControllerTrait"] = ControllerControlWidget
controls["Dict"] = DictControlWidget
controls["Compound"] = CompoundControlWidget
controls["List_File"] = OffscreenListFileControlWidget
controls["Range"] = range_editor
controls["unknown"] = NonEditableControlWidget
controls["Undefined"] = NonEditableControlWidget
# nipype types
controls["ImageFileSPM"] = FileControlWidget



# -*- coding: utf-8 -*-

#  This software and supporting documentation are distributed by
#      Institut Federatif de Recherche 49
#      CEA/NeuroSpin, Batiment 145,
#      91191 Gif-sur-Yvette cedex
#      France
#
# This software is governed by the CeCILL-B license under
# French law and abiding by the rules of distribution of free software.
# You can  use, modify and/or redistribute the software under the
# terms of the CeCILL-B license as circulated by CEA, CNRS
# and INRIA at the following URL "http://www.cecill.info".
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL-B license and that you accept its terms.


'''
Base classes for reading various minf formats (XML, HDF5, Python's pickle, etc.)

* author: Yann Cointepas
* organization: `NeuroSpin <http://www.neurospin.org>`_
* license: `CeCILL B <http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html>`_
'''
from __future__ import absolute_import
__docformat__ = "restructuredtext en"


import six
from soma.translation import translate as _

#------------------------------------------------------------------------------


class RegisterMinfReaderClass(type):

    '''
    RegisterMinfReaderClass is used as metaclass of L{MinfReader} to automatically
    register all classes derived from L{MinfReader}.
    '''
    def __init__(cls, name, bases, dict):
        # why test hasattr(cls, name) ?
        # on Ubuntu 12.04 the six.with_metaclass() function may trigger this
        # constructor on a "NewBase" type which doesn't have the name attribute
        if hasattr(cls, 'name') and cls.name is not None:
            MinfReader._allReaderClasses[cls.name] = cls


#------------------------------------------------------------------------------
class MinfReader(six.with_metaclass(RegisterMinfReaderClass, object)):

    '''
    Class derived from MinfReader are responsible of reading a specific format of
    minf file. This version only support XML format but other formats may be added
    later (such as HDF5).
    All classes derived from L{MinfReader} must define a L{name} class attribute
    and L{nodeIterator} method. The constructor of L{MinfReader} derived class
    must be callable without arguments (except C{self}), it does not have to be
    overloaded.
    '''

    #: all classes derived from L{MinfReader} are automatically stored in that
    #: dictionary (keys are formats name and values are class objects).
    _allReaderClasses = {}

    #: class derived from L{MinfReader} must set a format name in this attribute.
    name = None

    def createReader(format):
        '''
        This static method create a L{MinfReader} instance by looking for a registered
        L{MinfReader} derived classes named C{format}.
        @param format: name of the minf format.
        @type  format: string
        @returns: L{MinfReader} derived class instance.
        '''
        reader = MinfReader._allReaderClasses.get(format)
        if reader is None:
            raise ValueError(
                _('No minf reader for format "%(format)s", possible formats are: %(possible)s') %
                  {'format': format, 'possible': ', '.join(['"' + i + '"' for i in MinfReader._allReaderClasses])})
        return reader()
    createReader = staticmethod(createReader)

    def reduction(self, source):
        '''
        Return a pair C{( reduction, buffer )} where C{reduction} is the name of
        the L{MinfReducer<soma.minf.tree.MinfReducer} stored in the minf file and
        C{buffer} is the data that have been read from file.
        '''

    def nodeIterator(self, source):
        '''
        Return an iterator on all the nodes of the minf tree corresponding to the content
        of the C{source} minf file.
        @param source: source of the minf file.
        @type  source: string of file object
        '''



# -*- coding: utf-8 -*-
"""QImage <-> numpy.ndarray conversion module.

*** ATTENTION: This code is outdated - I released a better extension
*** named 'qimage2ndarray' (that completes all TODO items below) in
*** the meantime, which is available via PyPI and here:
*** http://kogs-www.informatik.uni-hamburg.de/~meine/software/qimage2ndarray/

This supports conversion in both directions; note that in contrast to
C++, in Python it is not possible to convert QImages into ndarrays
without copying the data.  The conversion functions in the opposite
direction however do not copy the data.

TODO:

- support record arrays in rgb2qimage
  (i.e. grok the output of qimage2numpy)
- support unusual widths/alignments also in gray2qimage and
  rgb2qimage
- make it possible to choose between views and copys of the data
  (eventually in both directions, when implemented in C++)
- allow for normalization in numpy->QImage conversion
  (i.e. to quickly visualize images with different value ranges)
- implement in C++
"""

from __future__ import absolute_import
import numpy
from soma.qt_gui.qt_backend.QtGui import QImage, QColor
from six.moves import range

bgra_dtype = numpy.dtype({'b': (numpy.uint8, 0),
                          'g': (numpy.uint8, 1),
                          'r': (numpy.uint8, 2),
                          'a': (numpy.uint8, 3)})


def qimage2numpy(qimage, dtype='array'):
    """Convert QImage to numpy.ndarray.  The dtype defaults to uint8
    for QImage.Format_Indexed8 or `bgra_dtype` (i.e. a record array)
    for 32bit color images.  You can pass a different dtype to use, or
    'array' to get a 3D uint8 array for color images."""
    result_shape = (qimage.height(), qimage.width())
    temp_shape = (qimage.height(),
                  qimage.bytesPerLine() * 8 / qimage.depth())
    if qimage.format() in (QImage.Format_ARGB32_Premultiplied,
                           QImage.Format_ARGB32,
                           QImage.Format_RGB32):
        if dtype == 'rec':
            dtype = bgra_dtype
        elif dtype == 'array':
            dtype = numpy.uint8
            result_shape += (4, )
            temp_shape += (4, )
    elif qimage.format() == QImage.Format_Indexed8:
        dtype = numpy.uint8
    else:
        raise ValueError("qimage2numpy only supports 32bit and 8bit images")
    # FIXME: raise error if alignment does not match
    buf = qimage.bits().asstring(qimage.numBytes())
    result = numpy.frombuffer(buf, dtype).reshape(temp_shape)
    if result_shape != temp_shape:
        result = result[:, :result_shape[1]]
    if qimage.format() == QImage.Format_RGB32 and dtype == numpy.uint8:
        result = result[..., :3]
    return result


def numpy2qimage(array):
    if numpy.ndim(array) == 2:
        return gray2qimage(array)
    elif numpy.ndim(array) == 3:
        return rgb2qimage(array)
    raise ValueError("can only convert 2D or 3D arrays")


def gray2qimage(gray):
    """Convert the 2D numpy array `gray` into a 8-bit QImage with a gray
    colormap.  The first dimension represents the vertical image axis.

    ATTENTION: This QImage carries an attribute `ndimage` with a
    reference to the underlying numpy array that holds the data. On
    Windows, the conversion into a QPixmap does not copy the data, so
    that you have to take care that the QImage does not get garbage
    collected (otherwise PyQt will throw away the wrapper, effectively
    freeing the underlying memory - boom!)."""
    if len(gray.shape) != 2:
        raise ValueError("gray2QImage can only convert 2D arrays")

    gray = numpy.require(gray, numpy.uint8, 'C')

    h, w = gray.shape

    result = QImage(gray.data, w, h, QImage.Format_Indexed8)
    result.ndarray = gray
    for i in range(256):
        result.setColor(i, QColor(i, i, i).rgb())
    return result


def rgb2qimage(rgb):
    """Convert the 3D numpy array `rgb` into a 32-bit QImage.  `rgb` must
    have three dimensions with the vertical, horizontal and RGB image axes.

    ATTENTION: This QImage carries an attribute `ndimage` with a
    reference to the underlying numpy array that holds the data. On
    Windows, the conversion into a QPixmap does not copy the data, so
    that you have to take care that the QImage does not get garbage
    collected (otherwise PyQt will throw away the wrapper, effectively
    freeing the underlying memory - boom!)."""
    if len(rgb.shape) != 3:
        raise ValueError("rgb2QImage can only convert 3D arrays")
    if rgb.shape[2] not in (3, 4):
        raise ValueError(
            "rgb2QImage can expects the last dimension to contain exactly three (R,G,B) or four (R,G,B,A) channels")

    h, w, channels = rgb.shape

    # Qt expects 32bit BGRA data for color images:
    bgra = numpy.empty((h, w, 4), numpy.uint8, 'C')
    bgra[..., 0] = rgb[..., 2]
    bgra[..., 1] = rgb[..., 1]
    bgra[..., 2] = rgb[..., 0]
    if rgb.shape[2] == 3:
        bgra[..., 3].fill(255)
        fmt = QImage.Format_RGB32
    else:
        bgra[..., 3] = rgb[..., 3]
        fmt = QImage.Format_ARGB32

    result = QImage(bgra.data, w, h, fmt)
    result.ndarray = bgra
    return result

if __name__ == '__main__':
    import pylab
    i = QImage()
#    i.load("qimage2ndarray_test.png")
    i.load("qimage2ndarray_test_lenna.jpg")
    v = qimage2numpy(i, "rec")
    v2 = qimage2numpy(i, "array")
    pylab.imshow(v2[..., 0])
    pylab.show()

    # v is a recarray; make it MPL-compatible for showing:
    rgb = numpy.empty(v.shape + (3, ), dtype=numpy.uint8)
    rgb[..., 0] = v["r"]
    rgb[..., 1] = v["g"]
    rgb[..., 2] = v["b"]
    pylab.imshow(rgb)
    pylab.show()



# -*- coding: utf-8 -*-
 
from __future__ import print_function

from __future__ import absolute_import
import sys
import unittest
import json
from soma.serialization import JSONSerializable, from_json


class TestSerializable(JSONSerializable):
    def __init__(self, a, b):
        self.a = a
        self.b = b
        
    # python3 does not support __cmp__()
    def __eq__(self, other):
        return self.a == other.a and self.b == other.b

    def __lt__(self, other):
        return (self.a, self.b) < (other.a, other.b)

    def to_json(self):
        return ['soma.tests.test_serialization.test_serializable',
                [self.a, self.b]]
    
    def __str__(self):
        return 'TestSerializable({0}, {1}'.format(self.a, self.b)
    
def test_serializable(a=None, b=None):
    return TestSerializable(a, b)


class TestJSONSerialization(unittest.TestCase):

    def test_instance_serialization(self):
        x = TestSerializable(12345, 67890)
        j = json.dumps(x.to_json())
        y = from_json(json.loads(j))
        self.assertEqual(x,y)


    def test_serialization_formats(self):
        y = from_json('soma.tests.test_serialization.test_serializable')
        self.assertEqual(y, TestSerializable(None, None))
        x = TestSerializable(12345, 67890)
        y = from_json(['soma.tests.test_serialization.test_serializable',
                                        [12345, 67890]])
        self.assertEqual(x, y)
        y = from_json(['soma.tests.test_serialization.test_serializable',
                                        {'a': 12345, 'b': 67890}])
        self.assertEqual(x, y)
        y = from_json(['soma.tests.test_serialization.test_serializable',
                                        [12345], {'b': 67890}])
        self.assertEqual(x, y)

    def test_serialization_failure(self):
        with self.assertRaises(ValueError):
            from_json('missing_dot')
        with self.assertRaises(ValueError):
            from_json('invalid.module.name')
        with self.assertRaises(ValueError):
            from_json('soma.tests.test_serialization.not_existing')

def test():
    suite = unittest.TestLoader().loadTestsFromTestCase(TestJSONSerialization)
    runtime = unittest.TextTestRunner(verbosity=2).run(suite)
    return runtime.wasSuccessful()


if __name__ == "__main__":
    test()



# -*- coding: utf-8 -*-
from __future__ import print_function
from __future__ import absolute_import
import os
import tempfile
import re
import sys
import soma.subprocess
try:
    import nbformat
    from jupyter_core.command import main as main_jupyter
except ImportError:
    print('cannot import nbformat and/or jupyter_core.command: cannot test '
          'notebooks')
    main_jupyter = None


def _notebook_run(path, output_nb, timeout=60):
    """Execute a notebook via nbconvert and collect output.
       :returns (parsed nb object, execution errors)

       from: http://blog.thedataincubator.com/2016/06/testing-jupyter-notebooks/
    """
    if main_jupyter is None:
        print('cannot test notebook', path)
        return None, []

    dirname, __ = os.path.split(path)
    old_cwd = os.getcwd()
    os.chdir(dirname)
    if not os.path.isabs(path):
        path = os.path.basename(path)
    ret_code = 1
    args = ["jupyter", "nbconvert", "--to", "notebook", "--execute",
      "--ExecutePreprocessor.timeout=%d" % timeout,
      "--ExecutePreprocessor.kernel_name=python%d" % sys.version_info[0],
      "--output", output_nb, path]
    old_argv = sys.argv
    sys.argv = args
    sys.argv[0] = re.sub(r'(-script\.pyw|\.exe)?$', '', sys.argv[0])

    try:
        ret_code = main_jupyter()
    finally:
        sys.argv = old_argv
        os.chdir(old_cwd)

    return ret_code


def notebook_run(path, timeout=60):
    """Execute a notebook via nbconvert and collect output.
       :returns (parsed nb object, execution errors)

       from: http://blog.thedataincubator.com/2016/06/testing-jupyter-notebooks/
    """
    if main_jupyter is None:
        print('cannot test notebook', path)
        return None, []

    dirname, __ = os.path.split(path)
    old_cwd = os.getcwd()
    os.chdir(dirname)
    nb = None
    fout = tempfile.mkstemp(suffix=".ipynb")
    os.close(fout[0])
    try:
        print('temp nb:', fout[1])
        args = [sys.executable, '-m', 'soma.test_utils.test_notebook',
                path, fout[1], str(timeout)]

        try:
            # call _notebook_run as an external process because it will
            # sys.exit()
            ret_code = soma.subprocess.call(args)

            nb = nbformat.read(fout[1], nbformat.current_nbformat)
        except Exception as e:
            print('EXCEPTION:', e)
            return None, [e]
    finally:
        try:
            os.unlink(fout[1])
        except OSError:
            pass
        os.chdir(old_cwd)

    errors = [output for cell in nb.cells if "outputs" in cell
                     for output in cell["outputs"]\
                     if output.output_type == "error"]

    return nb, errors


def test_notebook(notebook_filename, timeout=60):
    """Almost the same as notebook_run() but returns a single arror code

    Parameters
    ----------
    notebook_filename: filename of the notebook (.ipynb) to test
    timeout: int
        max time in seconds to execute one cell

    Returns
    -------
    code: True if successful, False if failed
    """
    if main_jupyter is None:
        raise Warning('cannot import nbformat and/or jupyter_core.command: '
                      'cannot test notebooks')

    print("running notebook test for", notebook_filename)
    nb, errors = notebook_run(notebook_filename, timeout)

    if len(errors) == 0:
        code = True
    else:
        code = False
    return code


if __name__ == '__main__':
    timeout=60
    if len(sys.argv) >=4:
        timeout = int(sys.argv[3])
    sys.exit(_notebook_run(sys.argv[1], sys.argv[2], timeout=timeout))



# -*- coding: utf-8 -*-

#  This software and supporting documentation are distributed by
#      Institut Federatif de Recherche 49
#      CEA/NeuroSpin, Batiment 145,
#      91191 Gif-sur-Yvette cedex
#      France
#
# This software is governed by the CeCILL-B license under
# French law and abiding by the rules of distribution of free software.
# You can  use, modify and/or redistribute the software under the
# terms of the CeCILL-B license as circulated by CEA, CNRS
# and INRIA at the following URL "http://www.cecill.info".
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL-B license and that you accept its terms.

'''
Some useful functions to manage file or directorie names.

* author: Yann Cointepas
* organization: NeuroSpin
* license: `CeCILL B <http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html>`_
'''
from __future__ import absolute_import
__docformat__ = "restructuredtext en"

import os
import platform
import fnmatch
import glob
import hashlib
import re
import shutil
import six
import sys


def split_path(path):
    '''
    Iteratively apply :func:`os.path.split` to build a list. Ignore trailing directory separator.

    Examples
    --------
    ::

        split_path('/home/myaccount/data/file.csv') returns ['/', 'home', 'myaccount', 'data', 'file.csv']
        split_path('home/myaccount/data/file.csv') returns ['home', 'myaccount', 'data', 'file.csv']
        split_path('/home/myaccount/data/') returns ['/', 'home', 'myaccount', 'data']
        split_path('/home/myaccount/data') returns ['/', 'home', 'myaccount', 'data']
        split_path('') returns ['']
  '''
    result = []
    a, b = os.path.split(path)
    if not b:
        a, b = os.path.split(a)
    while a and b:
        result.insert(0, b)
        a, b = os.path.split(a)
    if a:
        result.insert(0, a)
    else:
        result.insert(0, b)
    return result


def relative_path(path, referenceDirectory):
    '''
    Return a relative version of a path given a
    reference directory.

    ::

        os.path.join(referenceDirectory, relative_path(path, referenceDirectory))
        returns os.path.abspath(pat)

    Example
    =======
    ::

        relative_path('/usr/local/brainvisa-3.1/bin/brainvisa', '/usr/local')
        returns 'brainvisa-3.1/bin/brainvisa'

        relative_path('/usr/local/brainvisa-3.1/bin/brainvisa', '/usr/local/bin')
        returns '../brainvisa-3.1/bin/brainvisa'

        relative_path('/usr/local/brainvisa-3.1/bin/brainvisa', '/tmp/test/brainvisa')
        returns '../../../usr/local/brainvisa-3.1/bin/brainvisa'
    '''
    sPath = split_path(os.path.abspath(path))
    sReferencePath = split_path(os.path.abspath(referenceDirectory))
    i = 0
    while i < len(sPath) and i < len(sReferencePath) and sPath[i] == sReferencePath[i]:
        i += 1
    plist = (['..'] * (len(sReferencePath) - i)) + sPath[i:]
    if len(plist) == 0:
        return ''
    return os.path.join(*plist)


query_string_re = re.compile('\?([^\?\&]+\=[^\&]*)(\&[^\?\&]+\=[^\&]*)*$')


def split_query_string(path):
    '''
    Split a path and its query string.

    Example
    =======
    A path containing a query string is::

        /dir1/file1?param1=val1&param2=val2&paramN=valN

    ::

        split_query_string( '/dir1/file1?param1=val1&param2=val2&paramN=valN' )

    would return::

        ('/dir1/file1', '?param1=val1&param2=val2&paramN=valN')

    '''
    m = query_string_re.search(path)
    if m is not None:
        return (path[0:m.start()], path[m.start():])

    else:
        return (path, '')


def remove_query_string(path):
    '''
    Remove the query string from a path.

    Example
    =======
    A path containing a query string is::

        /dir1/file1?param1=val1&param2=val2&paramN=valN

    ::

        remove_query_string( '/dir1/file1?param1=val1&param2=val2&paramN=valN' )

    would return::

        '/dir1/file1'

    '''
    return query_string_re.sub('', path)


def strict_urlparse(path):
    '''
    A "fixed" version of urlparse.urlparse() which preserves the case of the
    drive letter in windows paths. The standard urlparse changes 'Z:/some/path'
    to 'z:/some/path'
    '''
    from six.moves.urllib import parse as urlparse
    url_parsed = urlparse.urlparse(path)
    if len(url_parsed.scheme) == 1 and url_parsed.scheme >= 'a' \
            and url_parsed.scheme <= 'z' and path[1] == ':' \
              and path[0] == url_parsed.scheme.upper():
        url_parsed = type(url_parsed)(
            scheme=url_parsed.scheme.upper(),
            netloc=url_parsed.netloc,
            path=url_parsed.path,
            params=url_parsed.params,
            query=url_parsed.query,
            fragment=url_parsed.fragment)
    return url_parsed

def parse_query_string(path):
    '''
    Parses the query string from a path and returns a dictionary.

    Example
    =======
    '''
    from six.moves.urllib import parse as urlparse
        
    url_parsed = urlparse.urlparse(path)
    qs_parsed = urlparse.parse_qs(url_parsed.query)
    
    return dict([(k, v[0]) if isinstance(v, list) and len(v) == 1 else (k, v) \
                 for k, v in six.iteritems(qs_parsed)])
    

class QueryStringParamUpdateMode(object):
    REPLACE = 0
    APPEND = 1
    REMOVE = 2

def update_query_string(
    path,
    params,
    params_update_mode=QueryStringParamUpdateMode.REPLACE
):
    '''
    Update the query string parameters in a path.

    Parameters
    ----------
    path: string
          The path to update parameters within.

    params: dict|list
          A dictionnary that contains keys and parameters to set in the query
          string

    params_update_mode: dict|string|list|int
          The default value is QueryStringParamUpdateMode.REPLACE that lead to
          replace value in the query string path by the one given in the params
          dictionary.

          It is possible to change the default behaviour giving the value
          QueryStringParamUpdateMode.APPEND. This will lead to always append
          values of the params dictionary to values of the query string path.
          The default behaviour can also be changed by specifying a parameter
          name as string, in this case only values for that parameter name will
          be appended. It can also contains a list or a tuple of parameter names
          for which values will be appended.

          Finally, this parameter can be a dictionary that specifies which
          parameter has to be appended or replaced. The dictionary contains
          parameter names in its keys and QueryStringParamUpdateMode in values.

    Returns
    -------
    path: string
          The path updated with given parameters

    Example
    -------
    A path containing a query string is::

        /dir1/file1?param1=val1&param2=val2&paramN=valN

    the params dictionary contains::

        {'param1':'newval1', param2=newval2', param3':'newval3'}

    ::

        update_query_string('/dir1/file1?param1=val1&param2=val2&paramN=valN',
                            {'param1':'newval1', 'param2':'newval2', 'param3':'newval3'})

    would return::

        '/dir1/file1?param1=newval1&param2=newval2&paramN=valN&param3=newval3'

    ::

        update_query_string('/dir1/file1?param1=val1&param2=val2&paramN=valN',
                            {'param1':'newval1', 'param2':'newval2', 'param3':'newval3'},
                            QueryStringParamUpdateMode.APPEND)

    would return::

        '/dir1/file1?param1=val1&param1=newval1&param2=val2&param2=newval2&paramN=valN&param3=newval3'

    ::

        update_query_string('/dir1/file1?param1=val1&param2=val2&paramN=valN',
                            {'param1':'newval1', 'param2':'newval2', 'param3':'newval3'},
                            'param2')

    would return::

        '/dir1/file1?param1=newval1&param2=val2&param2=newval2&paramN=valN&param3=newval3'

    ::

        update_query_string('/dir1/file1?param1=val1&param2=val2&paramN=valN',
                            {'param1':'newval1', 'param2':'newval2', 'param3':'newval3'},
                            ('param1', 'param2'))

    would return::

        '/dir1/file1?param1=val1&param1=newval1&param2=val2&param2=newval2&paramN=valN&param3=newval3'

    ::

        update_query_string('/dir1/file1?param1=val1&param2=val2&paramN=valN',
                            {'param1':'newval1', 'param2':'newval2', 'param3':'newval3'},
                            {'param1': QueryStringParamUpdateMode.APPEND,
                            'param2': QueryStringParamUpdateMode.REPLACE})

    would return::

        '/dir1/file1?param1=val1&param1=newval1&param2=val2&param2=newval2&paramN=valN&param3=newval3'
    '''
    from six.moves.urllib import parse as urllib
    urlparse = urllib

    # Convert params_update_mode to a dictionary that contains the update mode
    # for each parameter
    if type(params_update_mode) in (list, tuple):
        # Update mode is specified using a list of parameter names
        default_update_mode = QueryStringParamUpdateMode.REPLACE
        params_update = params_update_mode
        params_update_mode = dict()

        for p in params_update:
            if (type(p) in (list, tuple)):
                if (len(p) > 1):
                    params_update_mode[p[0]] = p[1]
                elif(len(p) > 0):
                    params_update_mode[
                        p[0]] = QueryStringParamUpdateMode.APPEND
            else:
                params_update_mode[p] = QueryStringParamUpdateMode.APPEND

    elif isinstance(params_update_mode, six.string_types):
        # A parameter name was given directly
        default_update_mode = QueryStringParamUpdateMode.REPLACE
        params_update_mode = dict(((params_update_mode,
                                    QueryStringParamUpdateMode.APPEND),))

    elif params_update_mode in (QueryStringParamUpdateMode.APPEND,
                                QueryStringParamUpdateMode.REPLACE,
                                QueryStringParamUpdateMode.REMOVE):
        # Update mode was specified for all parameters
        default_update_mode = params_update_mode
        params_update_mode = dict()

    elif type(params_update_mode) is dict:
        default_update_mode = QueryStringParamUpdateMode.REPLACE

    else:
        raise RuntimeError(
            'params_update_mode is not specified correctly. '
            'It must be either a dictionary that contains parameter names '
            'and the corresponding QueryStringParamUpdateMode, '
            'either a list that contains parameter names, either'
            'QueryStringParamUpdateMode.')

    url_parsed = strict_urlparse(path)
    url_params = urlparse.parse_qs(url_parsed.query)
    
    if isinstance(params, (list, tuple)):
        params = dict([(p, '') for p in params])
        

    # Update parameters dictionary
    for p, v in six.iteritems(params):
        update_mode = params_update_mode.get(
            p,
            default_update_mode
        )

        if update_mode == QueryStringParamUpdateMode.REPLACE:
            url_params[p] = v

        elif update_mode == QueryStringParamUpdateMode.APPEND:
            if type(v) in (list, tuple):
                if type(v) is tuple:
                    url_params[p] += list(v)
                else:
                    url_params[p] += v

            else:
                url_params.setdefault(p, list()).append(v)
                
        elif update_mode == QueryStringParamUpdateMode.REMOVE:
            del url_params[p]
            
        else:
            raise RuntimeError('params_update_mode is not specified correctly. %s is '
                               'not a valid value for parameter %s. Valid values are '
                               'either QueryStringParamUpdateMode.APPEND, either'
                               'QueryStringParamUpdateMode.REPLACE.' % (v, p))
        
        
    url_new = list(url_parsed)
    url_new[4] = urllib.urlencode(url_params, doseq=True)

    return urlparse.urlunparse(url_new)


def find_in_path(file, path=None):
    '''
    Look for a file in a series of directories. By default, directories are
    contained in ``PATH`` environment variable. But another environment
    variable name or a sequence of directories names can be given in *path*
    parameter.

    Examples::

      find_in_path('sh') could return '/bin/sh'
      find_in_path('libpython3.10.so', 'LD_LIBRARY_PATH') could return '/usr/lib/x86_64-linux-gnu/libpython3.10.so'
    '''
    if path is None:
        path = os.environ.get('PATH').split(os.pathsep)
    elif isinstance(path, six.string_types):
        var = os.environ.get(path)
        if var is None:
            var = path
        path = var.split(os.pathsep)
    for i in path:
        p = os.path.normpath(os.path.abspath(i))
        if p:
            r = glob.glob(os.path.join(p, file))
            if r:
                return r[0]


def locate_file(pattern, root=os.curdir):
    """
    Locates a file in a directory tree

    :param string pattern:
        The pattern to find

    :param string root:
        The search root directory

    :returns:
        The first found occurrence
    """
    for path, dirs, files in os.walk(os.path.abspath(root)):
        for filename in fnmatch.filter(files, pattern):
            return os.path.join(path, filename)


def update_hash_from_directory(directory, hash):
    '''
    Update a hash object from the content of a directory. The hash will
    reflect the recursive content of all files as well as the paths in all
    directories.
    '''
    for root, dirs, files in sorted(os.walk(directory)):
        for file in sorted(files):
            hash.update(file.encode('utf-8'))
            hash.update(open(os.path.join(root, file), 'rb').read())
        for dir in sorted(dirs):
            hash.update(dir.encode('utf-8'))
            update_hash_from_directory(os.path.join(root, dir), hash)


def path_hash(path, hash=None):
    '''
    Return a hash hexdigest for a file or a directory.
    '''
    if hash is None:
        hash = hashlib.md5()
    if os.path.isdir(path):
        update_hash_from_directory(path, hash)
    else:
        hash.update(open(path, 'rb').read())
    return hash.hexdigest()


def ensure_is_dir(d, clear_dir=False):
    """ If the directory doesn't exist, use os.makedirs """
    if not os.path.exists(d):
        os.makedirs(d)
    elif clear_dir:
        shutil.rmtree(d)
        os.makedirs(d)



# -*- coding: utf-8 -*-
#
# SOMA - Copyright (C) CEA, 2015
# Distributed under the terms of the CeCILL-B license, as published by
# the CEA-CNRS-INRIA. Refer to the LICENSE file or to
# http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html
# for details.
#

# System import
from __future__ import absolute_import
from __future__ import print_function
import logging
import os
import sys
import six

# Define the logger
logger = logging.getLogger(__name__)

# Soma import
from soma.qt_gui import qt_backend
from soma.qt_gui.qt_backend import QtGui, QtCore, Qt
from soma.controller import trait_ids
from soma.qt_gui.timered_widgets import TimeredQLineEdit
from soma.functiontools import partial
from soma.sorted_dictionary import OrderedDict
from soma.functiontools import SomaPartial
import weakref
from soma.utils.weak_proxy import get_ref, weak_proxy
import traits.api as traits
import inspect
import sip

# Qt import
try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    _fromUtf8 = lambda s: s

qt_backend.set_qt_backend(compatible_qt5=True)
# setup notification in Qt GUI thread
qt_backend.init_traitsui_handler()


class ScrollControllerWidget(Qt.QScrollArea):

    """ Class that create a widget to set the controller parameters.

    The widget is placed in a scroll area when large sets of parameters have
    to be tuned.
    """

    def __init__(self, controller, parent=None, name=None, live=False,
                 hide_labels=False, select_controls=None,
                 disable_controller_widget=False, override_control_types=None,
                 user_data=None, userlevel=0):
        """ Method to initilaize the ScrollControllerWidget class.

        Parameters
        ----------
        controller: derived Controller instance (mandatory)
            a class derived from the Controller class we want to parametrize
            with a widget.
        parent: QtGui.QWidget (optional, default None)
            the controller widget parent widget.
        name: (optional, default None)
            the name of this controller widget
        live: bool (optional, default False)
            if True, synchronize the edited values in the widget with the
            controller values on the fly,
            otherwise, wait synchronization instructions to update the
            controller values.
        hide_labels: bool (optional, default False)
            if True, don't show the labels associated with the controls
        select_controls: str (optional, default None)
            parameter to select specific conrtoller traits. Authorized options
            are 'inputs' or 'outputs'.
        disable_controller_widget: bool (optional, default False)
            if True disable the controller widget.
        override_control_types: dict (optional)
            if given, this is a "factory" dict assigning new controller editor
            types to some traits types.
        user_data: any type (optional)
            optional user data that can be accessed by individual control
            editors
        userlevel: int
            the current user level: some traits may be marked with a non-zero userlevel, and will only be visible if the ControllerWidget userlevel is more than (or equal) the trait level.
        """
        # Inheritance
        super(ScrollControllerWidget, self).__init__(parent)

        self.user_data = user_data

        # Allow the application to resize the scroll area items
        self.setWidgetResizable(True)
        self.setSizePolicy(QtGui.QSizePolicy.Preferred,
                           QtGui.QSizePolicy.Preferred)

        # Display a surounding box
        self.setFrameShape(QtGui.QFrame.StyledPanel)

        # Create the controller widget
        self.controller_widget = ControllerWidget(
            controller, parent, name, live, hide_labels, select_controls,
            override_control_types=override_control_types, user_data=user_data,
            userlevel=userlevel)
        self.controller_widget.layout().setContentsMargins(2, 2, 2, 2)
        self.controller_widget.setSizePolicy(QtGui.QSizePolicy.Expanding,
                                             QtGui.QSizePolicy.Preferred)

        # Enable / disabled the controller widget
        self.controller_widget.setEnabled(not disable_controller_widget)

        # Set the controller widget in the scroll area
        self.setWidget(self.controller_widget)

    def __del__(self):
        # disconnect underlying ControllerWidget so that it can be deleted.
        self.controller_widget.disconnect()

    @property
    def userlevel(self):
        if hasattr(self, 'controller_widget'):
            return self.controller_widget.userlevel
        return 0

    @userlevel.setter
    def userlevel(self, value):
        if hasattr(self, 'controller_widget'):
            self.controller_widget.userlevel = value

    def update_controller(self):
        self.controller_widget.update_controller()


class DeletableLineEdit(QtGui.QWidget):

    """ Close button + line editor, used for modifiable key labels
    """
    userModification = QtCore.Signal()
    buttonPressed = QtCore.Signal()

    def __init__(self, text=None, parent=None):
        super(DeletableLineEdit, self).__init__(parent)
        layout = QtGui.QHBoxLayout()
        self.setLayout(layout)
        delete_button = QtGui.QToolButton()
        layout.addWidget(delete_button)
        # Set the tool icons
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(
            _fromUtf8(":/soma_widgets_icons/delete")),
            QtGui.QIcon.Normal, QtGui.QIcon.Off)
        delete_button.setIcon(icon)
        self.line_edit = TimeredQLineEdit(text)
        layout.addWidget(self.line_edit)
        self.line_edit.userModification.connect(self.userModification)
        delete_button.pressed.connect(self.buttonPressed)

    def text(self):
        return self.line_edit.text()

    def setText(self, text):
        self.line_edit.setText(text)


class ControllerWidget(QtGui.QWidget):

    """ Class that create a widget to set the controller parameters.
    """

    # Parameter to store the mapping between the string trait descriptions and
    # the associated control classes
    _defined_controls = {}

    def __init__(self, controller, parent=None, name=None, live=False,
                 hide_labels=False, select_controls=None,
                 editable_labels=False, override_control_types=None,
                 user_data=None, userlevel=0):
        """ Method to initilaize the ControllerWidget class.

        Parameters
        ----------
        controller: derived Controller instance (mandatory)
            a class derived from the Controller class we want to parametrize
            with a widget.
        parent: QtGui.QWidget (optional, default None)
            the controller widget parent widget.
        name: (optional, default None)
            the name of this controller widget
        live: bool (optional, default False)
            if True, synchronize the edited values in the widget with the
            controller values on the fly,
            otherwise, wait the synchronization instruction to update the
            controller values.
        hide_labels: bool (optional, default False)
            if True, don't show the labels associated with the controls
        select_controls: str (optional, default None)
            parameter to select specific conrtoller traits. Authorized options
            are 'inputs' or 'outputs'.
        editable_labels: bool (optional, default False)
            if True, labels (trait keys) may be edited by the user, their
            modification will trigger a signal.
        override_control_types: dict (optional)
            if given, this is a "factory" dict assigning new controller editor
            types to some traits types.
        user_data: any type (optional)
            optional user data that can be accessed by individual control
            editors
        userlevel: int
            the current user level: some traits may be marked with a non-zero userlevel, and will only be visible if the ControllerWidget userlevel is more than (or equal) the trait level.
        """
        # Inheritance
        super(ControllerWidget, self).__init__(parent)

        self._userlevel = userlevel

        if override_control_types:
            # copy dict
            self._defined_controls = dict(self.__class__._defined_controls)
            self._defined_controls.update(override_control_types)

        QtCore.QResource.registerResource(os.path.join(os.path.dirname(
            os.path.dirname(__file__)), 'resources', 'widgets_icons.rcc'))

        # Class parameters
        if controller is None:
            raise ValueError('null controller')
        if controller is traits.Undefined:
            raise ValueError('undefined controller')
        self.controller = controller
        self.live = live
        self.hide_labels = hide_labels
        self.select_controls = select_controls
        # Parameter to store the connection status between the
        # controller widget and the controller
        self.connected = False
        # Parameter to store all the controller widget controls:
        # the keys correspond to the control name (a control name is
        # associated to a controller trait with the same name), the
        # dictionary elements are 4-uplets of the form (trait, control_class,
        # control_instance, control_label).
        self._controls = {}
        self._keys_connections = {}
        self.editable_labels = editable_labels
        self.user_data = user_data

        # If possilbe, set the widget name
        if name:
            self.setObjectName(name)

        # Create the layout of the controller widget
        # We will add all the controls to this layout
        self._grid_layout = QtGui.QGridLayout()
        self._grid_layout.setAlignment(QtCore.Qt.AlignTop)
        self._grid_layout.setSpacing(3)
        self._grid_layout.setContentsMargins(0, 0, 0, 0)
        self.setLayout(self._grid_layout)

        self._groups = OrderedDict()

        # Create all the layout controls associated with the controller values
        # we want to tune (ie the user traits)
        self._create_controls()
        self.connect_keys()

        # Start the event loop that check for wrong edited fields (usefull
        # when we work off line, otherwise the traits make the job but it is
        # still user friendly).
        self._check()

        # Set the synchrinization between this object and the input controller:
        # 1) synchronize the edited values in the widget with the controller
        # values on the fly
        if self.live:
            self.connect()

        # 2) initialize the controller widget with the controller values and
        # wait synchronization instructions to update the controller values.
        else:
            self.update_controller_widget()

    #
    # Public members
    #

    def is_valid(self):
        """ Check that all edited fields are correct.

        Returns
        -------
        valid: bool
            True if all the controller widget controls are correctly filled,
            False otherwise
        """
        # Initilaized the output
        valid = True

        # Go through all the controller widget controls
        for control_name, control_groups in six.iteritems(self._controls):
            for group_name, control in six.iteritems(control_groups):

                # Unpack the control item
                trait, control_class, control_instance, control_label = control

                # Call the current control specific check method
                valid = control_class.is_valid(control_instance)

                # Stop checking if a wrong control has been found
                if not valid:
                    break

        return valid

    @property
    def userlevel(self):
        return self._userlevel

    @userlevel.setter
    def userlevel(self, value):
        self._userlevel = value
        self.update_controls()

    def update_controller(self):
        """ Update the controller.

        At the end the controller traits values will match the controller
        widget user defined parameters.
        """
        # Go through all the controller widget controls
        for control_name, control_groups in six.iteritems(self._controls):
            for group_name, control in six.iteritems(control_groups):

                # Unpack the control item
                trait, control_class, control_instance, control_label = control

                # Call the current control specific update controller method
                control_class.update_controller(self, control_name,
                                                control_instance)

    def update_controller_widget(self):
        """ Update the controller widget.

        At the end the controller widget user editable parameters will match
        the controller traits values.
        """
        # Go through all the controller widget controls
        for control_name, control_groups in six.iteritems(self._controls):
            for group_name, control in six.iteritems(control_groups):

                # Unpack the control item
                trait, control_class, control_instance, control_label = control

                # Call the current control specific update controller widget
                # method
                control_class.update_controller_widget(self, control_name,
                                                      control_instance)

    def connect(self):
        """ Connect the controller trait and the controller widget controls

        At the end al control will be connected with the associated trait, and
        when a 'user_traits_changed' signal is emited, the controls are updated
        (ie, deleted if necessary).
        """
        # If the controller and controller widget are not yet connected
        if not self.connected:

            # Go through all the controller widget controls
            for control_name, control_groups in six.iteritems(self._controls):
                for group_name, control in six.iteritems(control_groups):

                    # Unpack the control item
                    trait, control_class, control_instance, control_label \
                        = control

                    # Call the current control specific connection method
                    logger.debug("Connecting control '{0}: {1}'...".format(
                        control_name, control_instance))
                    control_class.connect(self, control_name, control_instance)

            # Add an event connected with the 'user_traits_changed' controller
            # signal: update the controls
            self.controller.on_trait_change(
                self.update_controls, "user_traits_changed", dispatch='ui')

            # if 'visible_groups' is a trait, connect it to groups
            if self.controller.trait('visible_groups'):
                self.controller.on_trait_change(
                    self.groups_vibility_changed, 'visible_groups',
                    dispatch='ui')

            # Update the controller widget values
            self.update_controller_widget()

            # notifications should be removed when the GUI is destroyed
            # (on C++ side)
            self.destroyed.connect(
                partial(self.static_disconnect, self.controller,
                        self.update_controls, self.groups_vibility_changed,
                        self._controls))
            # Update the connection status
            self.connected = True

    def connect_keys(self):
        if not self.editable_labels or self._keys_connections:
            return
        keys_connect = {}
        # Go through all the controller widget controls
        for control_name, control_groups in six.iteritems(self._controls):
            for group_name, control in six.iteritems(control_groups):
                hook1 = partial(self.__class__._key_modified,
                                weakref.proxy(self), control_name)
                hook2 = partial(self.__class__._delete_key,
                                weakref.proxy(self), control_name)
                #control = self._controls[control_name]
                label_control = control[3]
                if isinstance(label_control, tuple):
                    label_control = label_control[0]
                label_control.userModification.connect(hook1)
                label_control.buttonPressed.connect(hook2)
                keys_connect[control_name] = (label_control, hook1, hook2)
        self._keys_connections = keys_connect

    @staticmethod
    def static_disconnect(controller, update_controls, groups_vibility_changed,
                          controls, widget):
        ''' disconnect() cannot be called at the right time.
        static_disconnect() is called via a Qt signat when the widget is
        destroyed. But the python part is already destroyed then.
        '''
        controller.on_trait_change(
            update_controls, "user_traits_changed", remove=True)

        # if 'visible_groups' is a trait, connect it to groups
        if controller.trait('visible_groups'):
            controller.on_trait_change(
                groups_vibility_changed, 'visible_groups', remove=True)

        # Go through all the controller widget controls
        for control_name, control_groups in six.iteritems(controls):
            for group_name, control in six.iteritems(control_groups):

                # Unpack the control item
                trait, control_class, control_instance, control_label \
                    = control

                # Call the current control specific disconnection method
                try:
                    control_class.disconnect(widget, control_name,
                                             control_instance)
                except Exception:
                    pass  # probably something already deleted

    def disconnect(self):
        """ Disconnect the controller trait and the controller widget controls
        """
        # If the controller and controller widget are connected
        if self.connected:

            # Remove the 'update_controls' event connected with the
            # 'user_traits_changed' controller signal
            self.controller.on_trait_change(
                self.update_controls, "user_traits_changed", remove=True)

            # if 'visible_groups' is a trait, connect it to groups
            if self.controller.trait('visible_groups'):
                self.controller.on_trait_change(
                    self.groups_vibility_changed, 'visible_groups',
                    remove=True)

            # Go through all the controller widget controls
            for control_name, control_groups in six.iteritems(self._controls):
                for group_name, control in six.iteritems(control_groups):

                    # Unpack the control item
                    trait, control_class, control_instance, control_label \
                        = control

                    # Call the current control specific disconnection method
                    try:
                        control_class.disconnect(self, control_name,
                                                 control_instance)
                    except Exception:
                        pass  # probably something already deleted

            # Update the connection status
            self.connected = False

    def disconnect_keys(self):
        for control_name, connections in six.iteritems(self._keys_connections):
            label_widget, hook1, hook2 = connections
            label_widget.userModification.disconnect(hook1)
            label_widget.buttonPressed.disconnect(hook2)
        self._keys_connections = {}

    def update_controls(self):
        """ Event to refresh controller widget controls and intern parameters.

        The refresh is done off line, ie. we need first to disconnect the
        controller and the controller widget.
        """
        # Get the controller traits
        user_traits = self.controller.user_traits()

        # Assess the refreshing is done off line
        was_connected = self.connected
        if was_connected:
            self.disconnect()
        self.disconnect_keys()

        # Go through all the controller widget controls
        to_remove_controls = []
        for control_name, control_groups in six.iteritems(self._controls):
            for group_name, control in six.iteritems(control_groups):

                # Message
                logger.debug(
                    "Check if we need to update '{0}': trait in '{1}' "
                    "different from '{2}'?".format(
                        control_name, control, user_traits.get(control_name)))

                # Unpack the control item
                trait, control_class, control_instance, control_label = control

                # If the the controller trait is different from the trait
                # associated with the control
                if user_traits.get(control_name) != trait:

                    # Close and schedule for deletation the control widget
                    control_instance.close()
                    control_instance.deleteLater()

                    # Close and schedule for deletation the control labels
                    if isinstance(control_label, tuple):
                        for label in control_label:
                            if not sip.isdeleted(label):
                                label.close()
                                label.deleteLater()
                    elif control_label:
                        if not sip.isdeleted(control_label):
                            control_label.close()
                            control_label.deleteLater()

                    # Store the controls to be removed
                    to_remove_controls.append(control_name)

        # Delete all dead controls from the class '_controls' intern parameter
        for control_name in to_remove_controls:
            logger.debug("Delete control '{0}'.".format(control_name))
            del self._controls[control_name]

        # Recreate all the layout controls associated with the controller
        # values we want to tune (ie the user_traits): this procedure check
        # if the control has not already been created.
        self._create_controls()

        # Restore the connection status
        if was_connected:
            self.connect()
        self.connect_keys()

        # Update the widget geometry
        self.updateGeometry()

    #
    # Private members
    #

    def _check(self):
        """ Check that all edited fields are correct.

        At the end the controls with wrong values will be colored in red.
        """
        # Go through all the controller widget controls
        for control_name, control_groups in six.iteritems(self._controls):
            for group_name, control in six.iteritems(control_groups):

                # Unpack the control item
                trait, control_class, control_instance, control_label = control

                # Call the current control specific check method
                control_class.check(control_instance)

    def _create_controls(self):
        """ Method that will create a control for each user trait of the
        controller.

        Controller trait parameters that cannot be maped to controls
        will not appear in the user interface.
        """
        # Select only the controller traits of interest
        all_traits = self.controller.user_traits()
        if self.select_controls is None:
            selected_traits = all_traits
        elif self.select_controls == "inputs":
            selected_traits = dict(
                (trait_name, trait)
                for trait_name, trait in six.iteritems(all_traits)
                if not trait.output)
        elif self.select_controls == "outputs":
            selected_traits = dict(
                (trait_name, trait)
                for trait_name, trait in six.iteritems(all_traits)
                if trait.output)
        else:
            raise Exception(
                "Unrecognized 'select_controls' option '{0}'. Valid "
                "options are 'inputs' or 'outputs'.".format(self.select_controls))

        # Go through all the controller user traits
        skipped = set(['visible_groups'])
        for trait_name, trait in six.iteritems(selected_traits):
            if trait_name in skipped:
                continue
            # Create the widget
            self.create_control(trait_name, trait)

    def create_control(self, trait_name, trait):
        """ Create a control associated to a trait.

        Parameters
        ----------
        trait_name: str (mandatory)
            the name of the trait from which we want to create a control. The
            control widget will share the same name
        trait: Trait (mandatory)
            a trait item
        """
        # Search if the current trait has already been processed
        control_groups = self._controls.get(trait_name)
        control_instances = []
        control_labels = []

        # If no control has been found in the class intern parameters
        if control_groups is None:

            # Call the search function that will map the trait type to the
            # corresponding control type
            control_class = self.get_control_class(trait)

            # If no control has been found, skip this trait and print
            # an error message. Note that the parameter will not be
            # accessible in the user interface.
            if control_class is None:
                logger.error(
                    "No control defined for trait '{0}': {1}. This "
                    "parameter will not be accessible in the "
                    "user interface.".format(trait_name, trait_ids(trait)))
                return

            # handle groups
            layouts = []
            groups = trait.groups
            if groups:
                for group in groups:
                    group_widget = self._groups.get(group)
                    if group_widget is None:
                        group_widget = self._create_group_widget(group)
                        self._groups[group] = group_widget
                    layouts.append(group_widget.hideable_widget.layout())
            else:
                layouts.append(self._grid_layout)

            group = None
            for i, layout in enumerate(layouts):
                if groups:
                    group = groups[i]
                control_instance, control_label \
                      = self._create_control_in_layout(trait_name, trait,
                                                       layout, group)
                control_instances.append(control_instance)
                if control_label:
                    if not isinstance(control_label, tuple):
                        control_label = [control_label]
                    control_labels += list(control_label)
                    if isinstance(control_label[0], QtGui.QLabel):
                        control_label[0].setTextInteractionFlags(
                            QtCore.Qt.TextSelectableByKeyboard |
                            QtCore.Qt.TextSelectableByMouse)
                    control_label[0].setContextMenuPolicy(
                        Qt.Qt.CustomContextMenu)
                    control_label[0].customContextMenuRequested.connect(
                        partial(self._label_context_menu,
                                weakref.ref(control_label[0]), trait_name))

        # Otherwise, the control associated with the current trait name is
        # already inserted in the grid layout, just unpack the values
        # contained in the private '_controls' class parameter
        else:
            for group, control in six.iteritems(control_groups):
                _, _, control_instance, control_label = control
                control_instances.append(control_instance)
                if control_label:
                    if isinstance(control_label, tuple):
                        control_labels += list(control_label)
                    else:
                        control_labels.append(control_label)

        # Each trait has a hidden property. Take care of this information
        hide = (getattr(trait, 'hidden', False)
                or getattr(trait, 'unused', False)
                or (getattr(trait, 'userlevel', 0) is not None
                    and trait.userlevel > self.userlevel))

        # Show/Hide the control and associated labels
        for control_instance in control_instances:
            control_instance.setVisible(not hide)
        for label in control_labels:
            if not sip.isdeleted(label):  # sometimes happens...
                label.setVisible(not hide)

    def _create_group_widget(self, group):
        group_widget = QtGui.QGroupBox()
        last_row = self._grid_layout.rowCount()
        self._grid_layout.addWidget(group_widget, last_row, 0,
                                    1, 2)
        lay1 = QtGui.QVBoxLayout()
        lay1.setContentsMargins(0, 0, 0, 0)
        lay2 = QtGui.QHBoxLayout()
        lay1.addLayout(lay2)
        lay2.setContentsMargins(10, 0, 0, 0)
        lay2.addWidget(QtGui.QLabel('<html><em>%s</em></html>' % group))
        lay2.addStretch(1)
        icon = QtGui.QIcon()
        icon.addPixmap(
            QtGui.QPixmap(_fromUtf8(":/soma_widgets_icons/nav_down")),
            QtGui.QIcon.Normal, QtGui.QIcon.Off)
        group_widget.fold_button = QtGui.QPushButton(icon, '')
        group_widget.fold_button.setFixedSize(30, 20)
        lay2.addWidget(group_widget.fold_button)
        widget = QtGui.QWidget()
        group_widget.setLayout(lay1)
        lay1.addWidget(widget)
        group_widget.hideable_widget = widget
        layout = QtGui.QGridLayout()
        widget.setLayout(layout)
        layout.setAlignment(QtCore.Qt.AlignTop)
        layout.setSpacing(3)
        layout.setContentsMargins(5, 5, 5, 5)
        group_widget.setAlignment(QtCore.Qt.AlignLeft)

        visible_groups = getattr(self.controller, 'visible_groups', set())
        if group in visible_groups:
            show = True
        else:
            show = False
        group_widget.hideable_widget.setVisible(show)

        if not show:
            icon = QtGui.QIcon()
            icon.addPixmap(
                QtGui.QPixmap(_fromUtf8(":/soma_widgets_icons/nav_right")),
                QtGui.QIcon.Normal, QtGui.QIcon.Off)
            group_widget.fold_button.setIcon(icon)

        #group_widget.fold_button.clicked.connect(SomaPartial(
            #self._toggle_group_visibility, group))
        # FIXME: if we use this, self gets deleted somewhere. This is not
        # normal.
        group_widget.fold_button.clicked.connect(partial(
            self.__class__._toggle_group_visibility, weak_proxy(self), group))

        return group_widget

    def _set_group_visibility(self, group, checked):
        group_widget = self._groups[group]
        group_widget.hideable_widget.setVisible(checked)
        icon = QtGui.QIcon()
        if checked:
            icon.addPixmap(
                QtGui.QPixmap(_fromUtf8(":/soma_widgets_icons/nav_down")),
                QtGui.QIcon.Normal, QtGui.QIcon.Off)
        else:
            icon.addPixmap(
                QtGui.QPixmap(_fromUtf8(":/soma_widgets_icons/nav_right")),
                QtGui.QIcon.Normal, QtGui.QIcon.Off)
        group_widget.fold_button.setIcon(icon)

    def _toggle_group_visibility(self, group, checked=False):
        visible_groups = getattr(self.controller, 'visible_groups', set())
        if group in visible_groups:
            show = False
            visible_groups.remove(group)
        else:
            show = True
            visible_groups.add(group)
        self._set_group_visibility(group, show)
        self.controller.visible_groups = visible_groups

    def _create_control_in_layout(self, trait_name, trait, layout, group=None):
        # Call the search function that will map the trait type to the
        # corresponding control type
        control_class = self.get_control_class(trait)
        ## FIXME: for now we use a hack for compound/either traits, until
        ## we write a "real" GUI for them
        #if isinstance(trait.trait_type, (traits.TraitCompound, traits.Either)):
            ## compound trait: use the 1st
            #trait = trait.handler.handlers[0].as_ctrait()
        # Create the control instance and associated label
        if self.editable_labels:
            label_class = DeletableLineEdit
        else:
            label_class = QtGui.QLabel
        control_instance, control_label = control_class.create_widget(
            self, trait_name, getattr(self.controller, trait_name),
            trait, label_class, user_data=self.user_data)
        control_class.is_valid(control_instance)

        # If the trait contains a description, insert a tool tip to the
        # control instance
        type_name = type(trait.trait_type).__name__.split('.')[-1]
        #protected = ''
        #if self.controller.is_parameter_protected(trait_name):
            #protected = ' - modified'
        tooltip = "<b>%s</b> (%s)" % (trait_name, type_name)
        if trait.desc:
            tooltip += "<b>:</b> " + trait.desc
        if control_label:
            if isinstance(control_label, tuple):
                control_label[0].setToolTip(tooltip)
            else:
                control_label.setToolTip(tooltip)
        else:
            control_instance.setToolTip(tooltip)

        # Get the last empty row in the grid layout
        # Trick: If the grid layout is empty check the element 0
        # (not realistic but te grid layout return None)
        last_row = layout.rowCount()
        widget_item = layout.itemAtPosition(last_row, 1)
        while widget_item is None and last_row > 0:
            last_row -= 1
            widget_item = layout.itemAtPosition(last_row, 1)
        last_row += 1

        # If the control has no label, append the control in a two
        # columns span area of the grid layout
        if control_label is None:

            # Grid layout 'addWidget' parameters: QWidget, int row,
            # int column, int rowSpan, int columnSpan
            layout.addWidget(
                control_instance, last_row, 0, 1, 2)

        # If the control has two labels, add a first row with the
        # labels (one per column), and add the control in
        # the next row of the grid layout in the second column
        elif isinstance(control_label, tuple):

            # Get the number of label
            nb_of_labels = len(control_label)

            # If more than two labels are detected, print an error message.
            # We actually consider only the two first labels and skip the
            # others
            if nb_of_labels > 2:
                logger.error("To many labels associated with control "
                              "'{0}': {1}. Only consider the two first "
                              "labels and skip the others".format(
                                  trait_name, control_label))

            # Append each label in different columns
            if not self.hide_labels:
                layout.addWidget(
                    control_label[0], last_row, 0)
            if nb_of_labels >= 2:
                layout.addWidget(control_label[1], last_row, 1)

            # Append the control in the next row in the second column
            layout.addWidget(control_instance, last_row + 1, 1)

        # Otherwise, append the label and control in two separate
        # columns of the grid layout
        else:

            # Append the label in the first column
            if not self.hide_labels:
                layout.addWidget(control_label, last_row, 0)

            # Append the control in the second column
            layout.addWidget(
                control_instance, last_row, 1, 1, 1)

        # Store some informations about the inserted control in the
        # private '_controls' class parameter
        # Keys: the trait names
        # Parameters: the trait - the control name - the control - and
        # the labels associated with the control
        self._controls.setdefault(trait_name, {})[group] = (
            trait, control_class, control_instance, control_label)

        return control_instance, control_label

    def _key_modified(self, old_key):
        """ Dict / open controller key modification callback
        """
        control_groups = self._controls[old_key]
        control_labels = []
        for group_name, control in six.iteritems(control_groups):
            print('group_name:', group_name, ', control:', control)
            if isinstance(control[3], list):
                control_labels += control[3]
            else:
                control_labels.append(control[3])
        key = None
        for control_label in control_labels:
            if hasattr(control_label, 'text'):
                key = str(control_label.text())
                was_connected = self.connected
                if was_connected:
                    self.disconnect()
                self.disconnect_keys()

        if key is None:
            print('Modified dict key widget cannot be found', file=sys.stderr)
            return

        controller = self.controller
        print('add trait on:', controller)
        trait = controller.trait(old_key)
        controller.add_trait(key, trait)
        setattr(controller, key, getattr(controller, old_key))
        controller.remove_trait(old_key)
        self._controls[key] = self._controls[old_key]
        del self._controls[old_key]
        if was_connected:
            self.connect()
        # reconnect label widget
        self.connect_keys()
        # self.update_controls()  # not even needed
        if hasattr(self, 'main_controller_def'):
            main_control_class, controller_widget, control_name, frame = \
                self.main_controller_def
            main_control_class.update_controller(
                controller_widget, control_name, frame)
        # self.update_controller()

    def _delete_key(self, key):
        """ Dict / open controller key deletion callback
        """
        controller = self.controller
        trait = controller.trait(key)
        controller.remove_trait(key)
        self.update_controls()
        if hasattr(self, 'main_controller_def'):
            main_control_class, controller_widget, control_name, frame = \
                self.main_controller_def
            main_control_class.update_controller(
                controller_widget, control_name, frame)
        # self.update_controller()

    def groups_vibility_changed(self):
        visible_groups = self.controller.visible_groups or set()
        for group, group_widget in six.iteritems(self._groups):
            if group in visible_groups:
                show = True
            else:
                show = False
            self._set_group_visibility(group, show)

    def get_control_class(self, trait):
        """ Find the control associated with the input trait.

        The mapping is defined in the global class parameter
        '_defined_controls'.

        Parameters
        ----------
        trait: Trait (mandatory)
            a trait item

        Returns
        -------
        control_class: class
            the control class associated with the input trait.
            If no match has been found, return None
        """
        # Initilaize the output variable
        control_class = None

        todo = [trait]
        done = set()
        while todo:
            trait = todo.pop(0)
            done.add(trait)

            trait_id = None
            ids = trait_ids(trait)
            if len(ids) >= 2:
                trait_id = 'Compound'
            elif len(ids) == 1:
                trait_id = ids[0]

            if trait_id is not None:
                control_class = self._defined_controls.get(trait_id)
                # Recursive construction: consider only the top level
                while control_class is None:
                    split_id = trait_id.rsplit("_", 1)
                    if len(split_id) == 1:
                        break
                    trait_id = split_id[0]
                    # Try to get the control class
                    control_class = self._defined_controls.get(trait_id)

                if control_class is not None:
                    break

            # not found: look in superclasses
            bases = trait.trait_type.__class__.__bases__ \
                + trait.__class__.__bases__
            for base in bases:
                if issubclass(base, traits.TraitType):
                    trait = self._instantiate_trait(base)
                    todo.append(trait)

        if control_class is None:
            # fallback to a label displaying "this value cannot be seen/edited"
            control_class = self._defined_controls.get('unknown')

        if inspect.isfunction(control_class):
            # the function may instantiate a specialized type dynamically
            control_class = control_class(trait)

        return control_class

    def _instantiate_trait(self, trait_type):
        if issubclass(trait_type, traits.BaseRange):
            return trait_type(0)
        return trait_type()

    def _label_context_menu(self, widget, control_name, pos):
        menu = Qt.QMenu(control_name, None)
        protect = menu.addAction('%s modified' % control_name)
        protect.setCheckable(True)
        protected = self.controller.is_parameter_protected(control_name)
        protect.setChecked(protected)

        menu.exec_(widget().mapToGlobal(pos))
        if protect.isChecked() != protected:
            self.controller.protect_parameter(control_name,
                                              protect.isChecked())


from soma.qt_gui.controls import controls

# Fill the controller widget mapping between the string trait descriptions and
# the associated control classes
ControllerWidget._defined_controls.update(controls)



# -*- coding: utf-8 -*-

from __future__ import print_function

from __future__ import absolute_import
import unittest
import os
import sys

from six.moves import reload_module


class TestUndefined(unittest.TestCase):

    def setUp(self):
        # disable traits module
        if 'traits' in sys.modules:
            self._traits = sys.modules['traits']
        else:
            self._traits = None
        if 'traits.api' in sys.modules:
            self._traits_api = sys.modules['traits.api']
        else:
            self._traits_api = None

    def restore_traits(self):
        # fix / restore traits module
        if self._traits_api is None:
            del sys.modules['traits.api']
        else:
            sys.modules['traits.api'] = self._traits_api
        if self._traits is None:
            del sys.modules['traits']
        else:
            sys.modules['traits'] = self._traits

    def tearDown(self):
        self.restore_traits()
        from soma import undefined
        reload_module(undefined)

    def test_undefined_builtin(self):
        sys.modules['traits'] = None
        sys.modules['traits.api'] = None
        from soma import undefined
        reload_module(undefined)
        self.assertTrue(hasattr(undefined, 'Undefined'))
        undef = undefined.Undefined
        self.assertTrue(isinstance(undef, undefined.UndefinedClass))
        self.assertEqual(repr(undef), '<undefined>')
        self.assertTrue(undefined.UndefinedClass.get_instance()
                        is undefined.Undefined)

    def test_undefined_traits(self):
        self.restore_traits()
        from soma import undefined
        from traits import trait_base
        import traits.api as traits
        reload_module(undefined)
        self.assertTrue(hasattr(undefined, 'Undefined'))
        undef = undefined.Undefined
        self.assertTrue(isinstance(undef, trait_base._Undefined))
        self.assertEqual(repr(undef), '<undefined>')


def test():
    suite = unittest.TestLoader().loadTestsFromTestCase(TestUndefined)
    runtime = unittest.TextTestRunner(verbosity=2).run(suite)
    return runtime.wasSuccessful()


if __name__ == "__main__":
    test()



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import print_function
import six

class GraphNode(object):
    """ Simple Graph Node Structure

    Attributes
    ----------
    name : str
        the node name
    meta : object
        a python object stored in the node
    links_to : list
         object to store the graph edges: sucessor
    links_from : list
        object to store the graph edges: predecessor
    links_to_degree : int
        degree of the node regarding the successors
    links_from_degree : int
        degree of the node regarding the predecessors

    Methods
    --------
    add_link_to
    remove_link_to
    add_link_from
    remove_link_from
    """

    def __init__(self, name, meta):
        """ Create a Graph Node

        Parameters
        ----------
        name: str (mandatory)
        the name of the node

        meta: object
        an python object to store in the node
        """
        self.name = name
        self.meta = meta
        # variables to store the graph edges
        self.links_to = []
        self.links_from = []
        # the degree of the node
        self.links_to_degree = 0
        self.links_from_degree = 0

    def add_link_to(self, node):
        """ Method to add a Successor

        Parameters
        ----------
        node: GraphNode (mandatory)
        the successor node
        """
        if node not in self.links_to:
            self.links_to.append(node)
            self.links_to_degree += 1

    def remove_link_to(self, node):
        """ Method to remove a Successor

        Parameters
        ----------
        node: GraphNode (mandatory)
        the successor node
        """
        if node in self.links_to:
            self.links_to.remove(node)
            self.links_to_degree -= 1

    def add_link_from(self, node):
        """ Method to add a Predecessor

        Parameters
        ----------
        node: GraphNode (mandatory)
        the predecessor node
        """
        if node not in self.links_from:
            self.links_from.append(node)
            self.links_from_degree += 1

    def remove_link_from(self, node):
        """ Method to remove a Predecessor

        Parameters
        ----------
        node: GraphNode (mandatory)
        the predecessor node
        """
        if node in self.links_from:
            self.links_from.remove(node)
            self.links_from_degree -= 1


class Graph(object):
    """ Simple Graph Structure on which we want to perform a
    topological tree (no cycle).

    The algorithm is based on the R.E. Tarjanlinear linear
    optimization (O(N+A)).

    Attributes
    ----------
    _nodes : dict
        the graph nodes {node.name: node}
    _links : set
        graph edges (from_node, to_node)

    Methods
    --------
    add_node
    find_node
    add_link
    topological_sort
    """

    def __init__(self):
        """ Create a Graph
        """
        self._nodes = {}
        self._links = set()

    def create_node(self, name, meta):
        """Create a new Graph node and adds it to the graph
 
         Parameters
        ----------
        name: str (mandatory)
        the name of the node

        meta: object
        an python object to store in the node
        """
        node = GraphNode(name, meta)
        self.add_node(node)
        return node
        
    def add_node(self, node):
        """ Method to add a GraphNode in the Graph

        Parameters
        ----------
        node: GraphNode (mandatory)
        the node to insert
        """
        if not isinstance(node, GraphNode):
            raise Exception("Expect a GraphNode, got {0}".format(node))
        if node.name in self._nodes:
            raise Exception("Expect a GraphNode with a unique name, "
                            "got {0}".format(node))
        self._nodes[node.name] = node

    def find_node(self, node_name):
        """ Method to find a GraphNode in the Graph

        Parameters
        ----------
        node_name: str (mandatory)
        the name of the desired node
        """
        if node_name in self._nodes:
            return self._nodes[node_name]
        return None

    def add_link(self, from_node, to_node):
        """ Method to add an edge between two GraphNodes of the Graph

        Parameters
        ----------
        from_node: GraphNode (mandatory)
        a node in the graph
        to_node: GraphNode (mandatory)
        the successor node
        """
        if from_node not in self._nodes:
            raise Exception("Node {0} is not defined in the Graph."
                   "Use add_node() method".format(from_node))
        if to_node not in self._nodes:
            raise Exception("Node {0} is not defined in the Graph."
                   "Use add_node() method".format(to_node))
        if (from_node, to_node) not in self._links:
            self._nodes[to_node].add_link_from(self._nodes[from_node])
            self._nodes[from_node].add_link_to(self._nodes[to_node])
            self._links.add((from_node, to_node))

    def topological_sort(self):
        """ Perform the topological sort: find an order in which all the
        nodes can be taken.
        Step 1: Identify nodes that have no incoming link (nnil).
        Step 2: Loop until there are nnil
        a) Delete the current nodes c_nnil of in-degree 0.
        b) Place it in the output.
        c) Remove all its outgoing links from the graph.
        d) If the node has in-degree 0, add the node to nnil.
        Step 3: Assert that there is no loop in the graph.

        Returns
        -------
        output: list of tuple
            a list of ordered nodes with a tuple element containing the node
            name and the node meta element.
        """
        ordered_nodes = []

        # Step 1
        nnil = []
        for name, node in six.iteritems(self._nodes):
            if node.links_from_degree == 0:
                nnil.append(node)

        # Step 2
        while len(nnil):
        #-- a
            c_nnil = nnil.pop()
        #-- b
            ordered_nodes.append(c_nnil)
        #-- c
            for node in c_nnil.links_to:
                node.remove_link_from(c_nnil)
        #-- d
                if node.links_from_degree == 0:
                    nnil.append(node)

        # Step 3
        if len(ordered_nodes) == len(self._nodes):
            return [(node.name, node.meta) for node in ordered_nodes]
        else:
            raise Exception("There is loop in the Graph."
                            "Please inverstigate")


if __name__ == '__main__':

    """ A toy example:
    slip -> chaussettes -> chemise -> veste -> pantalon -> ceinture ->
    chaussures -> cravate
    """

    objects = ["chaussures", "chaussettes", "slip", "pantalon", "ceinture",
        "chemise", "veste", "cravate"]

    dependencies = [
        ("slip", "pantalon"),
        ("chemise", "cravate"),
        ("chemise", "pantalon"),
        ("pantalon", "ceinture"),
        ("chaussettes", "chaussures"),
        ("pantalon", "chaussures"),
        ("ceinture", "chaussures"),
        ("chemise", "veste"),
        ]

    g = Graph()

    for o in objects:
        g.create_node(o, None)

    for d in dependencies:
        g.add_link(d[0], d[1])

    r = g.topological_sort()
    r = [x[0] for x in r]
    print(" -> ".join(r))



# -*- coding: utf-8 -*-
#
# SOMA - Copyright (C) CEA, 2015
# Distributed under the terms of the CeCILL-B license, as published by
# the CEA-CNRS-INRIA. Refer to the LICENSE file or to
# http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html
# for details.
#

from __future__ import print_function

# System import
from __future__ import absolute_import
import sys
import logging

# Trait import
from traits.api import *

# Soma import
from soma.qt_gui.qt_backend import QtGui, QtCore
from soma.qt_gui.controller_widget import (
    ControllerWidget, ScrollControllerWidget)

# Soma import
from soma.controller import Controller


class Point(Controller):
    x = Float()
    y = Float()


class TestControls(Controller):

    """ A dummy class to test all available controls.
    """

    # Global parameters
    # Traits we want to parametrized thanks to control widgets
    enum = Enum("1", "2", "3")
    i = Int()
    s = Str()
    f = Float()
    b = Bool()
    fp = File()
    dp = Directory()
    l = List(Float())
    ll = List(List(Float()))
    lll = List(List(List(Str())))

    def __init__(self):
        """" Initialize the TestControls class.
        """
        # Inheritance
        super(TestControls, self).__init__()

        # Set some default values
        self.l = [3.2, 0.5]
        self.ll = [[3.2, 0.5],  [1.1, 0.9]]
        self.lll = [[["a", "b", ""]], [["aa", "", "ff"]], [["s"]]]


if __name__ == "__main__":

    # Set the logging level
    logging.basicConfig(level=logging.INFO)

    # Create a qt applicaction
    app = QtGui.QApplication(sys.argv)

    # Create the controller we want to parametrized
    controller = TestControls()

    # Set some values to the controller parameters
    controller.s = ""
    controller.f = 10.2

    # Create to controller widget that are synchronized on the fly
    widget1 = ScrollControllerWidget(controller, live=True)
    widget2 = ControllerWidget(controller, live=True)
    widget1.show()
    widget2.show()

    # Check if the controller widget is valid before edition
    print("Controller widget valid before edition: ", end=' ')
    print(widget1.controller_widget.is_valid())

    # Start the qt loop
    app.exec_()

    # Check if the controller widget is valid after edition
    print("Controller widget valid after edition: ", end=' ')
    print(widget1.controller_widget.is_valid())



# -*- coding: utf-8 -*-

#  This software and supporting documentation are distributed by
#      Institut Federatif de Recherche 49
#      CEA/NeuroSpin, Batiment 145,
#      91191 Gif-sur-Yvette cedex
#      France
#
# This software is governed by the CeCILL-B license under
# French law and abiding by the rules of distribution of free software.
# You can  use, modify and/or redistribute the software under the
# terms of the CeCILL-B license as circulated by CEA, CNRS
# and INRIA at the following URL "http://www.cecill.info".
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL-B license and that you accept its terms.

'''
:py:class:`BufferAndFile` instances are used to read data from a file (for
instance for identification) and "put back" the data on the file.

- author: Yann Cointepas
- organization: NeuroSpin
- license: `CeCILL B <http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html>`_
'''
__docformat__ = 'restructuredtext en'

#------------------------------------------------------------------------------


class BufferAndFile(object):

    '''
    This class is a read only file-like object that allows to read ahead and
    push data back into the stream. All pushed back data are stored in a buffer
    that is "read" by all subsequent read acces until it is empty. When the
    buffer is empty, reading is done directly on the attached file object.

    Example::

      from soma.bufferandfile import BufferAndFile

      # Open a file with a buffer
      f = BufferAndFile.open(fileName)
      # Check that the file content is XML
      start = f.read(5)
      # Put back the read characters
      f.unread( start )
      if start == '<?xml':
          # Use the file in an XML parser
          ...
      elif start == '&HDF':
          # Use the file in an HDF5 parser
          ...

    '''

    def __init__(self, file_object):
        '''
        Create a file-like object that adds an :py:meth:`unread` method to an
        opened ``file_object``.
        '''
        super(BufferAndFile, self).__init__()
        self.__buffer = ''
        self.__file = file_object
        self.name = getattr(file_object, 'name', '<unknown>')

    def unread(self, string_value):
        '''
        Adds data at the begining of the internal buffer. Data in the internal
        buffer will be returned by all subsequent read acces until the buffer is empty.
        '''
        self.__buffer = string_value + self.__buffer

    def change_file(self, file_object):
        '''
        Change the internal file object (keeps the internal buffer untouched).
        '''
        self.__file = file_object

    def clone(self):
        '''
        Return a new L{BufferAndFile} instance with the same internale buffer and
        the same internal file object as C{self}.
        '''
        result = BufferAndFile(self.__file)
        result.__buffer = self.__buffer
        return result

    def read(self, size=None):
        '''
        Read the file
        '''
        if size is None:
            result = self.__buffer + self.__file.read()
        else:
            buffer_size = len(self.__buffer)
            if buffer_size >= size:
                result = self.__buffer[:size]
                self.__buffer = self.__buffer[size:]
            else:
                result = self.__buffer + self.__file.read(size - buffer_size)
                self.__buffer = ''
        return result

    def readline(self, size=None):
        '''
        Read one text line
        '''
        buffer_eol = self.__buffer.find('\n')
        if size is None:
            if buffer_eol < 0:
                result = self.__buffer + self.__file.readline()
                self.__buffer = ''
            else:
                buffer_eol += 1
                result = self.__buffer[:buffer_eol]
                self.__buffer = self.__buffer[buffer_eol:]
        else:
            if buffer_eol < 0:
                buffer_size = len(self.__buffer)
                if buffer_size >= size:
                    result = self.__buffer[:size]
                    self.__buffer = self.__buffer[size:]
                else:
                    result = self.__buffer + \
                        self.__file.readline(size - buffer_size)
                    self.__buffer = ''
            else:
                size = min(size, buffer_eol + 1)
                result = self.__buffer[:size]
                self.__buffer = self.__buffer[size:]
        return result

    def __iter__(self):
        '''
        Iteration protocol
        '''
        return self

    def next(self):
        '''
        Iteration protocol
        '''
        line = self.readline()
        if not line:
            raise StopIteration
        return line

    def tell(self):
        '''
        Position in file
        '''
        return self.__file.tell() - len(self.__buffer)

    def seek(self, offset, whence=0):
        '''
        If ``whence`` is 0 or 2 (absolute seek positioning) or if offset is
        negative, internal buffer is cleared and seek is done directly on the
        internal file object. Otherwise (relative seek with a positive offset),
        internal buffer is taken into account.
        '''
        if whence == 2 or whence == 0 or offset < 0:
            self.__buffer = ''
            return self.__file.seek(offset, whence)
        else:
            buflen = len(self.__buffer)
            if offset > buflen:
                self.__buffer = ''
                return self.__file.seek(offset - buflen, whence)
            else:
                self.__buffer = self.__buffer[offset:]

    def open(*args, **kwargs):
        '''
        Open a file with built-in :py:func:`python:open` and create a
        :py:class:`BufferAndFile` instance.
        '''
        return BufferAndFile(open(*args, **kwargs))
    open = staticmethod(open)
